﻿///////////////////////////////////////////////////////////////////////////////
// file ai.js
// Copyright (c) 2012 Frédéric J. Rézeau. All rights reserved.
///////////////////////////////////////////////////////////////////////////////

(function (namespace, undefined) {
    "use strict";

    function createEntity(levelManager, direction, positionX, scale) {
        var entity = new namespace.ZombieEntity(positionX, levelManager.maxY, 0.4);
        entity.direction = direction;
        entity.scaleFactor = scale;
        entity.hitPoints = 10;
        entity.maxHitPoints = 10;
        entity.armorCoefficient = 0.5;
        levelManager.entities.push(entity);
    }

    function createStrongEntity(levelManager, direction, positionX, scale) {
        var entity = new namespace.ZombieEntity(positionX, levelManager.maxY, 0.4);
        entity.direction = direction;
        entity.scaleFactor = scale;
        entity.hitPoints = 35;
        entity.maxHitPoints = 35;
        entity.armorCoefficient = 0;
        entity.damageFactor = 1.5;
        entity.headStyle = namespace.HeadStyleEnum.Bandaged;
        levelManager.entities.push(entity);
    }

    function createVeryStrongEntity(levelManager, direction, positionX, scale) {
        var entity = new namespace.ZombieEntity(positionX, levelManager.maxY, 0.4);
        entity.direction = direction;
        entity.scaleFactor = scale;
        entity.hitPoints = 60;
        entity.maxHitPoints = 60;
        entity.armorCoefficient = 0;
        entity.damageFactor = 3;
        entity.headStyle = namespace.HeadStyleEnum.Police;
        levelManager.entities.push(entity);
    }

    function createHighDamageEntity(levelManager, direction, positionX, scale) {
        var entity = new namespace.ZombieEntity(positionX, levelManager.maxY, 0.4);
        entity.direction = direction;
        entity.scaleFactor = scale;
        entity.hitPoints = 25;
        entity.maxHitPoints = 25;
        entity.armorCoefficient = 0.3;
        entity.damageFactor = 3;
        entity.headStyle = namespace.HeadStyleEnum.Punk;
        levelManager.entities.push(entity);
    }

    function createFastEntity(levelManager, direction, positionX, scale) {
        var entity = new namespace.ZombieEntity(positionX, levelManager.maxY, 0.4);
        entity.direction = direction;
        entity.scaleFactor = scale;
        entity.hitPoints = 20;
        entity.maxHitPoints = 20;
        entity.armorCoefficient = 0.3;
        entity.damageFactor = 2;
        entity.speed = 0.7;
        entity.baseSpeed = 0.7;
        entity.headStyle = namespace.HeadStyleEnum.Bandit;
        levelManager.entities.push(entity);
    }

    function createSlowEntity(levelManager, direction, positionX, scale) {
        var entity = new namespace.ZombieEntity(positionX, levelManager.maxY, 0.4);
        entity.direction = direction;
        entity.scaleFactor = scale;
        entity.hitPoints = 35;
        entity.maxHitPoints = 35;
        entity.armorCoefficient = 0.5;
        entity.speed = 0.45;
        entity.baseSpeed = 0.45;
        entity.headStyle = namespace.HeadStyleEnum.Tourist;
        levelManager.entities.push(entity);
    }

    // AI.
    namespace.AI = function () {
        if (!(this instanceof namespace.AI)) {
            throw new Error("Constructor called as a function.");
        }

        this.lastUpdate = 0;
        this.nextUpdateTime = 2;
        this.phase = 0;
        this.canStartWaves = false;
        this.absoluteTime = 0;
    };

    namespace.AI.prototype.update = function (elapsed, levelManager) {
        if (levelManager.primaryWeaponAssignmentDone && levelManager.secondaryWeaponAssignmentDone) {
            this.absoluteTime += elapsed;
        }

        if (this.absoluteTime < 3 && levelManager.primaryWeaponAssignmentDone && levelManager.secondaryWeaponAssignmentDone) {
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipFive;
        }
        else {
            levelManager.currentLevelTip = namespace.LevelTipEnum.None;
        }

        if (!this.canStartWaves && levelManager.currentLevel > 1 && (levelManager.leftPlayer.stack.objects.length > 0 || levelManager.rightPlayer.stack.objects.length > 0)) {
            // If the player has started building we can start the waves.
            this.canStartWaves = true;
        }

        // No AI update if level is still on cooldown or during weapon assignment.
        if (levelManager.levelStartCooldown === 0 && levelManager.primaryWeaponAssignmentDone && levelManager.secondaryWeaponAssignmentDone && levelManager.newItem === namespace.StackObjectTypeEnum.None) {
            switch (levelManager.currentLevel) {
                case 1:
                    this.updateLevelOne(elapsed, levelManager);
                    break;
                case 2:
                    this.updateLevelTwo(elapsed, levelManager);
                    break;
                case 3:
                    this.updateLevelThree(elapsed, levelManager);
                    break;
                case 4:
                    this.updateLevelFour(elapsed, levelManager);
                    break;
                case 5:
                    this.updateLevelFive(elapsed, levelManager);
                    break;
                case 6:
                    this.updateLevelSix(elapsed, levelManager);
                    break;
                case 7:
                    this.updateLevelSeven(elapsed, levelManager);
                    break;
                case 8:
                    this.updateLevelEight(elapsed, levelManager);
                    break;
                case 9:
                    this.updateLevelNine(elapsed, levelManager);
                    break;
                case 10:
                    this.updateLevelTen(elapsed, levelManager);
                    break;
                case 11:
                    this.updateLevelEleven(elapsed, levelManager);
                    break;
                case 12:
                    this.updateLevelTwelve(elapsed, levelManager);
                    break;
                case 13:
                    this.updateLevelThirteen(elapsed, levelManager);
                    break;
                case 14:
                    this.updateLevelFourteen(elapsed, levelManager);
                    break;
                case 15:
                    this.updateLevelFifteen(elapsed, levelManager);
                    break;
                case 16:
                    this.updateLevelSixteen(elapsed, levelManager);
                    break;
                case 17:
                    this.updateLevelSeventeen(elapsed, levelManager);
                    break;
                case 18:
                    this.updateLevelEighteen(elapsed, levelManager);
                    break;
                case 19:
                    this.updateLevelNineteen(elapsed, levelManager);
                    break;
                case 20:
                    this.updateLevelTwenty(elapsed, levelManager);
                    break;
                case 1000:
                    this.updateChallenge1(elapsed, levelManager);
                    break;
                case 1001:
                    this.updateChallenge2(elapsed, levelManager);
                    break;
                default:
                    this.updateTestLevel(elapsed, levelManager);
                    break;
            }
        }
    };

    namespace.AI.prototype.updateChallenge2 = function (elapsed, levelManager) {

        // Reset the tip.
        levelManager.currentLevelTip = namespace.LevelTipEnum.None;

        if (this.lastUpdate >= this.nextUpdateTime) {
            if (levelManager.mountedCrateCount > 1) {
                createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);
                createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
            }
        }

        if (this.phase === 8 && levelManager.entities.length < 3) {
            this.phase = 1;
            this.lastUpdate = 0;
            this.nextUpdateTime = 2;

            levelManager.daysCount++;
            namespace.getSavedData().daysCountHighScore = Math.max(namespace.getSavedData().daysCountHighScore, levelManager.daysCount);

            var playedEffect = false, effect1;
            if (levelManager.showHighScoreEffect && levelManager.daysCount > 0 && levelManager.daysCount > namespace.getSavedData().daysCountHighScore) {
                // Checkpoint effect.
                effect1 = new namespace.ProjectileEffect(levelManager.leftPlayer.x + (levelManager.rightPlayer.x - levelManager.leftPlayer.x) / 2, levelManager.maxY - 1, 0.5, 0, 0, -3, namespace.EffectTypeEnum.HighScore, 0);
                effect1.selfDestroyTime = 1.5;
                effect1.startFadeTime = 0.7;
                levelManager.effects.push(effect1);
                namespace.SoundManager.playHighScoreSound();
                levelManager.showHighScoreEffect = false;
                playedEffect = true;
            }

            if (!playedEffect) {
                // Checkpoint effect.
                effect1 = new namespace.ProjectileEffect(levelManager.leftPlayer.x + (levelManager.rightPlayer.x - levelManager.leftPlayer.x) / 2, levelManager.maxY - 1, 0.5, 0, 0, -3, namespace.EffectTypeEnum.Checkpoint, 0);
                effect1.selfDestroyTime = 1.5;
                effect1.startFadeTime = 0.7;
                levelManager.effects.push(effect1);
                namespace.SoundManager.playCheckpointSound();
            }

            createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.05);
            createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.7, 1.05);
            createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.05);
            createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.05);
        }

        switch (levelManager.daysCount + 6) {
            case 1:
                this.updateLevelOne(elapsed, levelManager);
                break;
            case 2:
                this.updateLevelTwo(elapsed, levelManager);
                break;
            case 3:
                this.updateLevelThree(elapsed, levelManager);
                break;
            case 4:
                this.updateLevelFour(elapsed, levelManager);
                break;
            case 5:
                this.updateLevelFive(elapsed, levelManager);
                break;
            case 6:
                this.updateLevelSix(elapsed, levelManager);
                break;
            case 7:
                this.updateLevelSeven(elapsed, levelManager);
                break;
            case 8:
                this.updateLevelEight(elapsed, levelManager);
                break;
            case 9:
                this.updateLevelNine(elapsed, levelManager);
                break;
            case 10:
                this.updateLevelTen(elapsed, levelManager);
                break;
            case 11:
                this.updateLevelEleven(elapsed, levelManager);
                break;
            case 12:
                this.updateLevelTwelve(elapsed, levelManager);
                break;
            case 13:
                this.updateLevelThirteen(elapsed, levelManager);
                break;
            case 14:
                this.updateLevelFourteen(elapsed, levelManager);
                break;
            case 15:
                this.updateLevelFifteen(elapsed, levelManager);
                break;
            case 16:
                this.updateLevelSixteen(elapsed, levelManager);
                break;
            case 17:
                this.updateLevelSeventeen(elapsed, levelManager);
                break;
            case 18:
                this.updateLevelEighteen(elapsed, levelManager);
                break;
            case 19:
                this.updateLevelNineteen(elapsed, levelManager);
                break;
            case 20:
                this.updateLevelTwenty(elapsed, levelManager);
                break;
            default:
                this.updateChallenge1(elapsed, levelManager);
                break;
        }
    };

    namespace.AI.prototype.updateChallenge1 = function (elapsed, levelManager) {

        // Reset the tip.
        levelManager.currentLevelTip = namespace.LevelTipEnum.None;

        if (!this.canStartWaves && (levelManager.leftPlayer.stack.objects.length > 0 || levelManager.rightPlayer.stack.objects.length > 0)) {
            // If the player has started building we can start the waves.
            this.canStartWaves = true;
        }
        else if (this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            if (this.absoluteTime < 3) {
                levelManager.currentLevelTip = namespace.LevelTipEnum.TipFive;
            }
            else {
                levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
            }
        }

        // Create the waves based on current phase.
        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime) {
                // Increase the pace as the kill count increases.
                var count = 2, strongHold = (levelManager.mountedCrateCount > 3);
                if (levelManager.killCount < 20) {
                    count = (Math.random() > 0.7) ? 3 : 2;
                    this.nextUpdateTime = 6;
                }
                else if (levelManager.killCount < 40) {
                    count = (Math.random() > 0.7) ? 4 : 2;
                    this.nextUpdateTime = 5;
                }
                else if (levelManager.killCount < 60) {
                    count = (Math.random() > 0.5) ? 4 : 2;
                    this.nextUpdateTime = 5;
                }
                else if (levelManager.killCount < 80) {
                    count = (Math.random() > 0.3) ? 4 : 2;
                    this.nextUpdateTime = 4;
                }
                else {
                    count = (Math.random() > 0.1) ? 4 : 2;
                    this.nextUpdateTime = 4;
                }

                var offset = 1;
                for (var i = 0; i < count; i += 1) {
                    var direction = (Math.random() > 0.5) ? namespace.DirectionEnum.Left : namespace.DirectionEnum.Right;
                    var rand = Math.random();
                    var position = 0;
                    if (namespace.DirectionEnum.Left === direction) {
                        position = levelManager.maxX + offset;
                    }
                    else {
                        position = levelManager.minX - offset;
                    }
                    if (rand > 0.95) {
                        if (strongHold) {
                            createVeryStrongEntity(levelManager, direction, position, 1);
                        }
                        else {
                            createSlowEntity(levelManager, direction, position, 1);
                        }
                    }
                    else if (rand > 0.85) {
                        if (strongHold) {
                            createVeryStrongEntity(levelManager, direction, position, 1);
                        }
                        else {
                            createFastEntity(levelManager, direction, position, 1);
                        }
                    }
                    else if (rand > 0.8) {
                        createVeryStrongEntity(levelManager, direction, position, 1);
                    }
                    else if (rand > 0.7) {
                        createHighDamageEntity(levelManager, direction, position, 1);
                    }
                    else if (rand > 0.6) {
                        createStrongEntity(levelManager, direction, position, 1);
                    }
                    else {
                        if (strongHold) {
                            createVeryStrongEntity(levelManager, direction, position, 1);
                        }
                        else {
                            createEntity(levelManager, direction, position, 1);
                        }
                    }
                    offset += 1;
                }

                namespace.SoundManager.playZombieComing();
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelTwenty = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 3, 1.1);
                        this.nextUpdateTime = 15;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 20;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelNineteen = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 3, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 4, 1.1);
                        this.nextUpdateTime = 15;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 20;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelEighteen = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 3, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 4, 1.1);
                        this.nextUpdateTime = 15;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 20;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelSeventeen = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 3, 1.1);
                        this.nextUpdateTime = 15;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 20;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelSixteen = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 3, 1.1);
                        this.nextUpdateTime = 15;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 20;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelFifteen = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 15;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 20;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelFourteen = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createSlowEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createSlowEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 2;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createSlowEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createSlowEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createSlowEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createSlowEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);

                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createSlowEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createSlowEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelThirteen = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 2;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelTwelve = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 2;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelEleven = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 2;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelTen = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 2;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createSlowEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelNine = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 2;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 4, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelEight = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 2;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 4, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelSeven = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 4;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createFastEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createFastEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelSix = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 4;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createVeryStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelFive = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 4;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);

                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.1, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelFour = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 4;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.9, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelThree = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }
        else if (this.phase > 2 && (levelManager.leftPlayer.stack.objects.length + levelManager.rightPlayer.stack.objects.length) < 4) {
            // Bring up a quick reminder about crates building.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipTwo;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 4;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.4, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.4, 1.1);

                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.5, 1);

                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelTwo = function (elapsed, levelManager) {

        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }
        else if (this.phase > 2 && (levelManager.leftPlayer.stack.objects.length + levelManager.rightPlayer.stack.objects.length) < 4) {
            // Bring up a quick reminder about crates building.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipTwo;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1.1);
                        this.nextUpdateTime = 4;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.2, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 2.1, 1.1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1.7, 1.1);
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2.1, 1);
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1.7, 1.1);
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

    namespace.AI.prototype.updateLevelOne = function (elapsed, levelManager) {
        var i, len;
        switch (levelManager.currentLevelStage) {
            case namespace.LevelStageEnum.TutorialZombiesComing:
                // Create the first wave.
                if (levelManager.entities.length === 0) {
                    createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 4.2, 1);
                    createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 5, 1.1);
                    namespace.SoundManager.playZombieComing();
                }
                break;
            case namespace.LevelStageEnum.TutorialHitBuildButton:
            case namespace.LevelStageEnum.TutorialHitCrateButton:
            case namespace.LevelStageEnum.TutorialBuildingFirstCrate:
            case namespace.LevelStageEnum.TutorialTapToContinue:
            case namespace.LevelStageEnum.TutorialTurnRight:
                // Pause the entities.
                for (i = 0, len = levelManager.entities.length; i < levelManager.entities.length; i += 1) {
                    levelManager.entities[i].moveCooldown = 0.2;
                }
                break;
            case namespace.LevelStageEnum.TutorialMoreZombiesComing:
                // Create the second wave.
                if (levelManager.entities.length === 0) {
                    levelManager.survivalPoints = 40;
                    createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.2, 1);
                    createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 3.5, 1);
                    createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 5, 1.1);
                    createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 6, 1.1);
                    createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 7, 1);
                    namespace.SoundManager.playZombieComing();
                }
                break;
            case namespace.LevelStageEnum.TutorialFirstWave:
                // Move to next stage when all entities are dead.
                if (levelManager.entities.length === 0) {
                    levelManager.currentLevelStage = namespace.LevelStageEnum.TutorialFirstWaveDone;
                }
                break;
            case namespace.LevelStageEnum.TutorialFirstWaveFinishJob:
                // Move to next stage when all entities are dead.
                if (levelManager.entities.length === 0) {
                    levelManager.currentLevelStage = namespace.LevelStageEnum.TutorialSecondWaveDone;
                }
                break;
            case namespace.LevelStageEnum.TutorialThirdWave:
                // Create the last wave.
                if (levelManager.entities.length === 0) {
                    levelManager.survivalPoints = 60;
                    createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1 + Math.random() * 0.15);
                    createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 2, 1 + Math.random() * 0.15);
                    createHighDamageEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 3, 1 + Math.random() * 0.15);
                    createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 4, 1 + Math.random() * 0.15);
                    createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 4, 1 + Math.random() * 0.15);
                    createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 4.7, 1 + Math.random() * 0.15);
                    levelManager.currentLevelStage = namespace.LevelStageEnum.None;
                    namespace.SoundManager.playZombieComing();
                }
                break;
            default:
                break;
        }

        this.lastUpdate += elapsed;
    };

    namespace.AI.prototype.updateTestLevel = function (elapsed, levelManager) {
        if (this.absoluteTime > 3 && this.phase === 0 && levelManager.leftPlayer.stack.objects.length === 0 && levelManager.rightPlayer.stack.objects.length === 0) {
            // If nothing is built then we display tip one.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipOne;
        }
        else if (levelManager.entities.length < 4 && this.phase === 7) {
            // Announce the big wave.
            levelManager.currentLevelTip = namespace.LevelTipEnum.TipThree;
        }

        if (this.canStartWaves) {
            if (this.lastUpdate > this.nextUpdateTime || levelManager.entities.length === 0) {
                switch (this.phase) {
                    case 0:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        this.nextUpdateTime = 4;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 1:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 2:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 3:
                        createStrongEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1.1);
                        this.nextUpdateTime = 10;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 4:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        this.nextUpdateTime = 12;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 5:
                        createEntity(levelManager, namespace.DirectionEnum.Left, levelManager.maxX + 1, 1);
                        this.nextUpdateTime = 18;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 6:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        this.nextUpdateTime = 35;
                        namespace.SoundManager.playZombieComing();
                        break;
                    case 7:
                        createEntity(levelManager, namespace.DirectionEnum.Right, levelManager.minX - 1, 1);
                        namespace.SoundManager.playZombieComing();
                        break;
                }

                this.phase += 1;
                this.lastUpdate = 0;
            }

            this.lastUpdate += elapsed;
        }
    };

})(window.ZCJGame = window.ZCJGame || {});
