﻿///////////////////////////////////////////////////////////////////////////////
// file licensemanager.js
// Copyright (c) 2012 Frédéric J. Rézeau. All rights reserved.
///////////////////////////////////////////////////////////////////////////////

(function (namespace, undefined) {
    "use strict";

    var isTrial = false;
    var useWindowsSimulator = false;

    var hasExternalNotify = (typeof window.external !== "undefined" && typeof window.external.notify !== "undefined");
    var isWindows = typeof Windows !== "undefined";

    var windowsApp, windowsLicense;

    namespace.setTrialLicense = function (value) {
        isTrial = (value === "false") ? false : true;
    };

    namespace.isTrialLicense = function () {
        return isTrial;
    };

    namespace.buyGame = function () {
        if (hasExternalNotify) {
            window.external.notify("commandBuyGame");
        }
        else if (isWindows) {
            var appPackageFamilyName = Windows.ApplicationModel.Package.current.id.familyName;
            var uriToLaunch = "ms-windows-store:PDP?PFN=" + appPackageFamilyName;
            var uri = new Windows.Foundation.Uri(uriToLaunch);
            Windows.System.Launcher.launchUriAsync(uri);
        }
    };

    function showReviewMsg() {
        var msg = new Windows.UI.Popups.MessageDialog("Please feel free to rate this game if you like it.\n\nPress 'OK' if you wish to rate the game now, press 'Cancel' to continue playing.", "We greatly welcome your feedback.");
        msg.commands.append(new Windows.UI.Popups.UICommand("OK", reviewMsgHandler, 0));
        msg.commands.append(new Windows.UI.Popups.UICommand("Cancel", reviewMsgHandler, 1));
        msg.cancelCommandIndex = 1;
        msg.defaultCommandIndex = 0;
        msg.showAsync();
    }

    function reviewMsgHandler(command) {
        switch (command.id) {
            case 0:
                var appPackageFamilyName = Windows.ApplicationModel.Package.current.id.familyName;
                var uriToLaunch = "ms-windows-store:REVIEW?PFN=" + appPackageFamilyName;
                var uri = new Windows.Foundation.Uri(uriToLaunch);
                Windows.System.Launcher.launchUriAsync(uri);
                break;
            case 1:
                break;
        }
    }

    namespace.reviewGame = function () {
        if (hasExternalNotify) {
            window.external.notify("commandReviewGame");
        }
        else if (isWindows) {
            showReviewMsg();
        }
    };

    namespace.buyItem = function (id) {
        CocoonJS.Store.purchaseProduct(id);
        //namespace.addGold(1000);
    };

    function updateLicense() {
        isTrial = windowsLicense.isTrial;
    }

    if (hasExternalNotify) {
        window.external.notify("commandGetTrialLicense");
    }
    else if (isWindows) {
        windowsApp = useWindowsSimulator ? Windows.ApplicationModel.Store.CurrentAppSimulator : Windows.ApplicationModel.Store.CurrentApp;
        windowsLicense = windowsApp.licenseInformation;
        isTrial = windowsLicense.isTrial;
        windowsLicense.addEventListener("licensechanged", updateLicense);
    }

})(window.ZCJGame = window.ZCJGame || {});


CocoonJS.Store.onProductsFetchStarted.addEventListener(function () {
});

CocoonJS.Store.onProductsFetchFailed.addEventListener(function () {
});

CocoonJS.Store.onProductsFetchCompleted.addEventListener(function (products) {
    for (var i = products.length - 1; i >= 0; i--) {
        CocoonJS.Store.addProduct(products[i]);
    };
});

CocoonJS.Store.onProductPurchaseStarted.addEventListener(function (productId) {
});

CocoonJS.Store.onProductPurchaseFailed.addEventListener(function (productId, err) {
});

CocoonJS.Store.onProductPurchaseCompleted.addEventListener(function (purchaseInfo) {
    switch (purchaseInfo.productId) {
        case "001":
            window.ZCJGame.addGold(1000);
            break;
        case "002":
            window.ZCJGame.addGold(1500);
            break;
        case "003":
            window.ZCJGame.addGold(2000);
            break;
        case "004":
            window.ZCJGame.addGold(3000);
            break;
        case "005":
            window.ZCJGame.addGold(5000);
            break;
        case "006":
            window.ZCJGame.addGold(10000);
            break;
    }
    CocoonJS.Store.consumePurchase(purchaseInfo.transactionId, purchaseInfo.productId);
    CocoonJS.Store.finishPurchase(purchaseInfo.transactionId);
});









function setTrialLicense(value) {
    "use strict";
    window.ZCJGame.setTrialLicense(value);
}

function setData(value) {
    "use strict";
    window.ZCJGame.loadData(value);
}

function onNavigateBack() {
    "use strict";
    var navBackActions = { "Stay": false, "Exit": true };
    if (window.ZCJGame.getShowProducts()) {
        window.ZCJGame.setShowProducts(false);
        return navBackActions.Stay;
    }
    else if (window.ZCJGame.getShowBoosters()) {
        window.ZCJGame.setShowBoosters(false);
        window.ZCJGame.showMainScreen();
        return navBackActions.Stay;
    }
    else if (window.ZCJGame.getShowCredits() === true || window.ZCJGame.getCurrentGameState() === window.ZCJGame.GameStateEnum.GameLogo || window.ZCJGame.getCurrentGameState() === window.ZCJGame.GameStateEnum.GameIntro || window.ZCJGame.getCurrentGameState() === window.ZCJGame.GameStateEnum.GameMenu || window.ZCJGame.getCurrentGameState() === window.ZCJGame.GameStateEnum.GameLoading) {
        if (window.ZCJGame.getShowCredits() === true) {
            window.ZCJGame.setShowCredits(false);
            return navBackActions.Stay;
        }
        else if (window.ZCJGame.getCurrentGameState() === window.ZCJGame.GameStateEnum.GameMenu) {
            if (window.ZCJGame.getCurrentMenuType() !== window.ZCJGame.MenuTypeEnum.ShowMain && window.ZCJGame.getCurrentMenuType() !== window.ZCJGame.MenuTypeEnum.None) {
                window.ZCJGame.showMainScreen();
                return navBackActions.Stay;
            }
            else {
                return navBackActions.Exit;
            }
        }
        else {
            return navBackActions.Exit;
        }
    }
    else if(window.ZCJGame.getCurrentLevel() > 5 && window.ZCJGame.isTrialLicense()) {
        window.ZCJGame.showMainScreen();
        return navBackActions.Stay;
    }

    window.ZCJGame.displayMenuConfirmDialog();
    return navBackActions.Stay;
}
