﻿///////////////////////////////////////////////////////////////////////////////
// file renderer.js
// Copyright (c) 2012 Frédéric J. Rézeau. All rights reserved.
///////////////////////////////////////////////////////////////////////////////

(function (namespace, undefined) {
    "use strict";

    // Precomputed offsets for 1024x1024 tiles.
    var OFFSET0X, OFFSET1X, OFFSET2X, OFFSET3X, OFFSET4X, OFFSET5X, OFFSET6X, OFFSET7X, OFFSET8X, OFFSET9X, OFFSET10X;

    var achievementToImageMap = [
        { "x": 0, "y": 0 },
        { "x": 0, "y": 1 },
        { "x": 0, "y": 2 },
        { "x": 0, "y": 3 },
        { "x": 0, "y": 4 },
        { "x": 0, "y": 5 },
        { "x": 0, "y": 6 },
        { "x": 0, "y": 7 },
        { "x": 4, "y": 0 },
        { "x": 4, "y": 1 },
        { "x": 4, "y": 2 },
        { "x": 4, "y": 3 },
        { "x": 4, "y": 4 },
        { "x": 4, "y": 5 },
        { "x": 4, "y": 6 }
    ];

    var drawInteractiveMap = false;

    namespace.setupTileOffsets = function () {
        var scaleUnit = namespace.getScaleUnit();
        var tileSize = (1 / scaleUnit);
        OFFSET0X = 0;
        OFFSET1X = tileSize * 1;
        OFFSET2X = tileSize * 2;
        OFFSET3X = tileSize * 3;
        OFFSET4X = tileSize * 4;
        OFFSET5X = tileSize * 5;
        OFFSET6X = tileSize * 6;
        OFFSET7X = tileSize * 7;
        OFFSET8X = tileSize * 8;
        OFFSET9X = tileSize * 9;
        OFFSET10X = tileSize * 10;
    };

    namespace.drawIntro = function (context) {
        var maxIntroTime = namespace.getMaxIntroTime();
        var introTime = namespace.getIntroTime();

        var heartbeat1 = namespace.getCarHeartbeat();
        var heartbeat2 = namespace.getWeaponHeartbeat();

        // Retrieve the projected hud dimensions.
        var canvasWidth = namespace.getCanvasWidth();
        var canvasHeight = namespace.getCanvasHeight();

        // Draw the background.
        context.drawImage(namespace.Resources.backgroundImage2, 0, 0, canvasWidth, canvasHeight);

        // Draw the sky element.
        context.drawImage(namespace.Resources.backdrop01Image, OFFSET0X, OFFSET4X, OFFSET2X, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET5X, OFFSET5X);

        context.save();
        context.translate(canvasWidth / 2, canvasHeight / 2);
        context.scale(1.4, 1.4);
        var offsetX = -OFFSET6X * 2 + ((maxIntroTime - introTime) * OFFSET1X * 0.3);
        for (var i = 0; i < 10; i += 1) {
            context.drawImage(namespace.Resources.backdrop01Image, OFFSET0X, OFFSET2X, OFFSET6X, OFFSET2X, offsetX, -OFFSET1X, OFFSET6X, OFFSET2X);
            offsetX += OFFSET6X;
        }
        context.restore();

        var alternate = false;
        var offsetX = -OFFSET7X * 2 + ((maxIntroTime - introTime) * OFFSET1X * 0.6);
        for (var i = 0; i < 10; i += 1) {
            alternate = !alternate;
            if (alternate) {
                context.drawImage(namespace.Resources.backdrop02Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, offsetX + OFFSET1X, OFFSET1X * 1.5, OFFSET1X, OFFSET2X);
                context.drawImage(namespace.Resources.backdrop02Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, offsetX, OFFSET2X, OFFSET1X, OFFSET2X);
            }
            else {
                context.drawImage(namespace.Resources.backdrop02Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, offsetX, OFFSET1X * 1.5, OFFSET1X, OFFSET2X);
            }
            offsetX += OFFSET7X;
        }

        // Draw the floor.
        context.save();
        context.fillStyle = "rgb(180, 134, 89)";
        context.fillRect(OFFSET0X, OFFSET4X, canvasWidth, canvasHeight);
        var alternate = false;
        var offsetX = -OFFSET7X * 2 + ((maxIntroTime - introTime) * OFFSET1X);
        for (var i = 0; i < 10; i += 1) {
            alternate = !alternate;
            if (alternate) {
                if (i === 2) {
                    context.drawImage(namespace.Resources.effectsImage, OFFSET6X, OFFSET1X, OFFSET3X, OFFSET2X, offsetX - OFFSET1X * 0.2, OFFSET1X * 1.4, OFFSET2X, OFFSET1X * 1.5);
                    context.drawImage(namespace.Resources.effectsImage, OFFSET6X, OFFSET3X, OFFSET3X, OFFSET3X, offsetX, OFFSET2X, OFFSET2X, OFFSET2X);
                }

                context.drawImage(namespace.Resources.backdrop01Image, OFFSET0X, OFFSET0X, OFFSET10X, OFFSET2X, offsetX, OFFSET3X, OFFSET10X, OFFSET2X);
            }
            else {
                context.drawImage(namespace.Resources.backdrop01Image, OFFSET0X, OFFSET0X, OFFSET10X, OFFSET2X, offsetX, OFFSET3X, OFFSET10X, OFFSET2X);
            }
            offsetX += OFFSET7X;
         }
         context.restore();

        var carX = canvasWidth / 2 - OFFSET3X;
        if (introTime > maxIntroTime - 1) {
            carX += (1 - (maxIntroTime - introTime)) * OFFSET6X;
        }
        else if (introTime < 1) {
            carX -= (1 - introTime) * OFFSET6X;
        }

        context.save();
        context.translate(carX, canvasHeight / 2 - OFFSET2X);
        context.scale(0.7, 0.7);

        var x = carX + OFFSET5X;
        context.save();
        context.globalAlpha = Math.min(0.3, heartbeat2.time);
        context.translate(x + OFFSET1X, canvasHeight / 2 + OFFSET1X);
        context.rotate(heartbeat1.time * Math.PI);
        context.translate(-(x + OFFSET1X), -(canvasHeight / 2 + OFFSET1X));
        context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, x, canvasHeight / 2, OFFSET2X, OFFSET2X);
        context.restore();

        var x = carX + OFFSET5X;
        context.save();
        context.globalAlpha = Math.min(0.1, heartbeat1.time);
        context.translate(x + OFFSET1X, canvasHeight / 2 + OFFSET1X );
        context.rotate(-heartbeat1.time * Math.PI);
        context.translate(-(x + OFFSET1X), -(canvasHeight / 2 + OFFSET1X));
        context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, x, canvasHeight / 2, OFFSET2X, OFFSET2X);
        context.restore();

        var x = carX + OFFSET1X * 5.5;
        context.save();
        context.globalAlpha = Math.min(0.1, heartbeat1.time);
        context.translate(x + OFFSET1X, canvasHeight / 2 + OFFSET1X);
        context.rotate(-heartbeat1.time * Math.PI);
        context.translate(-(x + OFFSET1X), -(canvasHeight / 2 + OFFSET1X));
        context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, x, canvasHeight / 2, OFFSET2X, OFFSET2X);
        context.restore();

        var x = carX + OFFSET6X;
        context.save();
        context.globalAlpha = Math.min(0.3, heartbeat2.time);
        context.translate(x + OFFSET1X, canvasHeight / 2 + OFFSET1X);
        context.rotate(-heartbeat2.time * Math.PI);
        context.translate(-(x + OFFSET1X), -(canvasHeight / 2 + OFFSET1X));
        context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, x, canvasHeight / 2, OFFSET2X, OFFSET2X);
        context.restore();

        var x = carX + OFFSET1X * 6.3;
        context.save();
        context.globalAlpha = Math.min(0.3, heartbeat2.time);
        context.translate(x + OFFSET1X, canvasHeight / 2 + OFFSET1X);
        context.rotate(-heartbeat1.time * Math.PI);
        context.translate(-(x + OFFSET1X), -(canvasHeight / 2 + OFFSET1X));
        context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, x, canvasHeight / 2, OFFSET2X, OFFSET2X);
        context.restore();

        context.drawImage(namespace.Resources.entitiesImage, OFFSET3X, OFFSET5X, OFFSET1X, OFFSET1X, carX + OFFSET1X * 3.7, canvasHeight / 2 + OFFSET1X * 0.9, OFFSET1X, OFFSET1X);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET5X, OFFSET6X, OFFSET3X, carX, canvasHeight / 2 - OFFSET1X - (heartbeat1.time * OFFSET1X) * 0.1, OFFSET6X, OFFSET3X);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET3X, OFFSET5X, OFFSET1X, OFFSET1X, carX + OFFSET1X * 0.5, canvasHeight / 2 + OFFSET1X * 0.9, OFFSET1X, OFFSET1X);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET1X, OFFSET5X, OFFSET2X, OFFSET1X, carX + OFFSET1X * 1.7, canvasHeight / 2 - OFFSET1X * 0.25 - (heartbeat2.time * OFFSET1X) * 0.1, OFFSET2X, OFFSET1X);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET7X, OFFSET4X, OFFSET2X, OFFSET1X, carX + OFFSET1X * 3.7, canvasHeight / 2 - OFFSET1X * 0.1 + (heartbeat2.time * OFFSET1X) * 0.1, OFFSET2X, OFFSET1X);
        context.restore();

        var alternate = false;
        var offsetX = -OFFSET7X * 2 + ((maxIntroTime - introTime) * OFFSET2X);
        for (var i = 0; i < 10; i += 1) {
            alternate = !alternate;
            if (alternate) {
                context.drawImage(namespace.Resources.backdrop02Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, offsetX, OFFSET1X * 2.7, OFFSET1X * 1.7, OFFSET2X * 1.7);
                context.drawImage(namespace.Resources.backdrop02Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, offsetX + OFFSET1X, OFFSET4X, OFFSET1X * 1.7, OFFSET2X * 1.7);
            }
            offsetX += OFFSET7X;
        }

        context.fillStyle = "rgb(0, 0, 0)";
        context.fillRect(0, 0, canvasWidth, OFFSET1X * 1.07);
        context.fillRect(0, canvasHeight - OFFSET1X * 1.07, canvasWidth, OFFSET1X * 1.07);
        context.fillStyle = "#990000";
        context.fillRect(0, 0, canvasWidth, OFFSET1X);
        context.fillRect(0, canvasHeight - OFFSET1X, canvasWidth, OFFSET1X);

        if (introTime < maxIntroTime - 2 && introTime > maxIntroTime - 6) {
            context.drawImage(namespace.Resources.effectsImage, OFFSET2X, OFFSET6X, OFFSET3X, OFFSET2X, 0, 0, OFFSET3X, OFFSET2X);
            context.drawImage(namespace.Resources.effectsImage, OFFSET5X, OFFSET6X, OFFSET5X, OFFSET1X, OFFSET3X, 0, OFFSET5X, OFFSET1X);
        }
        else if (introTime < maxIntroTime - 6) {
            context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET6X, OFFSET2X, OFFSET2X, canvasWidth - OFFSET2X, 0, OFFSET2X, OFFSET2X);
            context.drawImage(namespace.Resources.effectsImage, OFFSET5X, OFFSET7X, OFFSET5X, OFFSET1X, canvasWidth - OFFSET1X * 7.5, 0, OFFSET5X, OFFSET1X);
        }

        context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET8X, OFFSET5X, OFFSET2X, OFFSET1X, OFFSET8X, OFFSET5X, OFFSET2X, OFFSET1X);

        if (introTime < 1.2) {
            context.globalAlpha = 1 - (introTime - 0.2);
            context.fillStyle = "rgb(0, 0, 0)";
            context.fillRect(0, 0, canvasWidth, canvasHeight);
            context.globalAlpha = 1;
        }
    };

    namespace.drawLoader = function (context, hideProgress) {
        var gameLoadingData = namespace.getGameLoadingData();

        // Retrieve the projected hud dimensions.
        var canvasWidth = namespace.getCanvasWidth();
        var canvasHeight = namespace.getCanvasHeight();

        var size = namespace.screenToHud({ "x": canvasWidth, "y": canvasHeight });

        // Draw the background image.
        context.drawImage(namespace.Resources.loadingScreenImage, OFFSET0X, OFFSET0X, OFFSET10X, OFFSET1X * 6.5, 0, 0, canvasWidth, canvasHeight);

        // Apply the hud cameras to draw hud items.
        context.save();
        namespace.applyHudCameras();

        // Draw the progress.
        if (!hideProgress) {
            var totalWidth = 4;
            var maxLogoTime = 3;
            var filledWidth = gameLoadingData.current * totalWidth / gameLoadingData.total;
            context.drawImage(namespace.Resources.loadingScreenImage, OFFSET5X, OFFSET7X, OFFSET4X, OFFSET1X, size.x / 2 - 2, size.y / 2, 4, 1);

            context.save();
            context.beginPath();
            context.rect(size.x / 2 - 2, size.y / 2, filledWidth, 1);
            context.closePath();
            context.clip();
            context.drawImage(namespace.Resources.loadingScreenImage, OFFSET5X, OFFSET8X, OFFSET4X, OFFSET1X, size.x / 2 - 2, size.y / 2, 4, 1);
            context.restore();
        }

        //context.drawImage(namespace.Resources.loadingScreenImage, OFFSET5X, OFFSET9X, OFFSET4X, OFFSET1X, size.x / 2 - 2, size.y - 1, 4, 1);

        // Draw the logo.
        context.drawImage(namespace.Resources.loadingScreenImage, OFFSET0X, OFFSET7X, OFFSET3X, OFFSET2X, size.x / 2 - (3 / 2), size.y / 2 - (2 / 2) - 0.5, 3, 2);

        context.restore();
    };

    namespace.drawProducts = function (context, levelManager) {
        // Retrieve the projected hud dimensions.
        var canvasWidth = namespace.getCanvasWidth();
        var canvasHeight = namespace.getCanvasHeight();
        var size = namespace.screenToHud({ "x": canvasWidth, "y": canvasHeight });

        context.save();
        context.translate(-levelManager.productTime * OFFSET1X * 10, 0);
        context.drawImage(namespace.Resources.productsScreenImage, 0, 0, canvasWidth, canvasHeight);
        context.restore();

       // namespace.applyHudCameras();
      //  context.fillStyle = "rgba(255, 0, 0, 0.5)";
       // context.fillRect(1.3, 0.4, 2.5, 2.6);
      //  context.fillRect(3.9, 0.4, 2.5, 2.6);
      //  context.fillRect(6.5, 0.4, 2.5, 2.6);
      //  context.fillRect(1.3, 3.1, 2.5, 2.6);
      //  context.fillRect(3.9, 3.1, 2.5, 2.6);
       // context.fillRect(6.5, 3.1, 2.5, 2.6);

        context.restore();
    };

    namespace.drawBoosters = function (context, levelManager) {
        // Retrieve the projected hud dimensions.
        var canvasWidth = namespace.getCanvasWidth();
        var canvasHeight = namespace.getCanvasHeight();

        var menuPageTime = namespace.getMenuPageTime();

        var size = namespace.screenToHud({ "x": canvasWidth, "y": canvasHeight });

        // Draw the background image.
        context.drawImage(namespace.Resources.boostersScreenImage, 0, 0, canvasWidth, canvasHeight);

        // Apply the hud cameras to draw hud items.
        context.save();
        namespace.applyHudCameras();

        if (levelManager.boosters[0].activated) {
            context.drawImage(namespace.Resources.objectsImage, OFFSET7X, OFFSET5X, OFFSET2X, OFFSET2X, 3, 1.9, 1, 1);
        }
        if (levelManager.boosters[1].activated) {
            context.drawImage(namespace.Resources.objectsImage, OFFSET7X, OFFSET5X, OFFSET2X, OFFSET2X, 6.2, 1.9, 1, 1);
        }
        if (levelManager.boosters[2].activated) {
            context.drawImage(namespace.Resources.objectsImage, OFFSET7X, OFFSET5X, OFFSET2X, OFFSET2X, 3, 4.9, 1, 1);
        }
        if (levelManager.boosters[3].activated) {
            context.drawImage(namespace.Resources.objectsImage, OFFSET7X, OFFSET5X, OFFSET2X, OFFSET2X, 6.2, 4.9, 1, 1);
        }

        if (levelManager.boosters[0].free) {
            context.drawImage(namespace.Resources.objectsImage, OFFSET7X, OFFSET7X, OFFSET2X, OFFSET1X, 4, 2, 2, 1);
            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X, OFFSET6X, OFFSET1X, OFFSET1X, 4 + namespace.getTitleHeartbeat().time * 0.2, 1 + namespace.getTitleHeartbeat().time * 0.5, 1, 1);
        }
        if (levelManager.boosters[1].free) {
            context.drawImage(namespace.Resources.objectsImage, OFFSET7X, OFFSET7X, OFFSET2X, OFFSET1X, 7.2, 2, 2, 1);
            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X, OFFSET6X, OFFSET1X, OFFSET1X, 7.2 + namespace.getTitleHeartbeat().time * 0.2, 1 + namespace.getTitleHeartbeat().time * 0.5, 1, 1);
        }
        if (levelManager.boosters[2].free) {
            context.drawImage(namespace.Resources.objectsImage, OFFSET7X, OFFSET7X, OFFSET2X, OFFSET1X, 4, 5, 2, 1);
            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X, OFFSET6X, OFFSET1X, OFFSET1X, 4 + namespace.getTitleHeartbeat().time * 0.2, 4 + namespace.getTitleHeartbeat().time * 0.5, 1, 1);
        }
        if (levelManager.boosters[3].free) {
            context.drawImage(namespace.Resources.objectsImage, OFFSET7X, OFFSET7X, OFFSET2X, OFFSET1X, 7.2, 5, 2, 1);
            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X, OFFSET6X, OFFSET1X, OFFSET1X, 7.2 + namespace.getTitleHeartbeat().time * 0.2, 4 + namespace.getTitleHeartbeat().time * 0.5, 1, 1);
        }

      //  context.fillStyle = "rgba(255, 0, 0, 0.5)";
      //  context.fillRect(0.4, 0.7, 2.4, 2);
      //  context.fillRect(0.4, 3.7, 2.4, 2);

     //   context.fillRect(3.2, 0, 3, 3);
     //   context.fillRect(6.3, 0, 3, 3);
     //   context.fillRect(3.2, 3.2, 3, 3);
     //   context.fillRect(6.3, 3.2, 3, 3);

       // context.fillRect(9.3, 2.3, 1.5, 1.5);

        // Draw the gold amount.
        var gold = namespace.getSavedData().gold;
        var x = 0;
        if (gold > 999999) {
            x -= 0.9;
        }
        else if (gold > 9999) {
            x -= 0.6;
        }
        else if (gold > 99) {
            x -= 0.3;
        }
        namespace.drawNumber(context, gold.toString(), 1.4 + x, 1.8, 1);

        // Draw the lives count.
        var lives = namespace.getSavedData().lives;
        namespace.drawNumber(context, lives.toString(), 1.3, 4, 1 + namespace.getTitleHeartbeat().time * 0.2);

        // Draw the lives timer.
        var remain = namespace.getSavedData().refillTime > 0 ? (namespace.getSavedData().refillTime - (new Date().getTime() / 1000)) : 0;
        if (remain > 60) {
            remain = remain / 60 + 1;
        }
        else if(namespace.getSavedData().refillTime > 0){
            remain = 1;
        }
        namespace.drawNumber(context, (remain).toFixed(0), 1.95, 3.2, 0.7);

        // Draw the refill lives message.
        if (levelManager.refillMessageTime > 0) {
            context.globalAlpha = Math.min(1, levelManager.refillMessageTime);
            context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET2X, size.x / 2 - 2, size.y / 2 - 1, 4, 2);
            context.drawImage(namespace.Resources.objectsImage, OFFSET0X, OFFSET7X, OFFSET3X, OFFSET1X, size.x / 2 - 1.5, size.y /2 - 0.5, 3, 1);
            context.globalAlpha = 1;
        }

        context.restore();
    };

    namespace.draw = function (context, levelManager) {
        var i, len, position, effect, directionOffset, levelItem, pageIndex;

        // Retrieve the game state and data for this frame.
        var currentGameState = namespace.getCurrentGameState();

        // Retrieve the projected hud dimensions.
        var canvasWidth = namespace.getCanvasWidth();
        var canvasHeight = namespace.getCanvasHeight();
        var size = namespace.screenToHud({ "x": canvasWidth, "y": canvasHeight });

        var menuHudItems = namespace.getMenuHudItems();
        var levelHudItems = namespace.getLevelHudItems();

        if (currentGameState === namespace.GameStateEnum.GameLogo) {
            namespace.drawLoader(context, true);
        }
        else if (currentGameState === namespace.GameStateEnum.GameIntro) {
            // Draw the intro screen.
            namespace.drawIntro(context);
        }
        else if (currentGameState === namespace.GameStateEnum.GameMenu) {
            var menuSetupTime = namespace.getMenuSetupTime();
            var maxMenuSetupTime = namespace.getMaxMenuSetupTime();
            var menuEntities = namespace.getMenuEntities();
            var menuPageTime = namespace.getMenuPageTime();
            var menuPageDirection = namespace.getMenuPageDirection();

            // Draw the menu screen background.
            if (menuSetupTime > maxMenuSetupTime - 1) {
                context.globalAlpha = maxMenuSetupTime - menuSetupTime;
            }
            context.drawImage(namespace.Resources.menuScreenImage, 0, 0, canvasWidth, canvasHeight);
            context.globalAlpha = 1;

            // Apply the hud cameras for drawing.
            namespace.applyHudCameras();

            // Draw the menu entities.
            for (i = 0, len = menuEntities.length; i < len; i += 1) {
                namespace.drawEntity(context, menuEntities[i]);
            }

            // Draw the title.
            position = { "x": 0.5, "y": 0.2 };
            var heartbeat1 = namespace.getWeaponHeartbeat();
            if (menuSetupTime > maxMenuSetupTime - 0.5 && namespace.getCurrentMenuType() === namespace.MenuTypeEnum.None) {
                position.x -= (0.5 - (maxMenuSetupTime - menuSetupTime)) * 2;
                position.y += (0.5 - (maxMenuSetupTime - menuSetupTime)) * 2;
            }
            context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET0X, OFFSET0X, OFFSET4X, OFFSET3X, position.x, position.y, 4, 3);

            // Draw the heroes.
            if (namespace.getCurrentMenuType() === namespace.MenuTypeEnum.None || namespace.getCurrentMenuType() === namespace.MenuTypeEnum.ShowMain) {
                position = { "x": 5, "y": 2.4 };
                if (menuSetupTime > maxMenuSetupTime - 0.5) {
                    position.y += (0.5 - (maxMenuSetupTime - menuSetupTime)) * 2;
                }
                context.save();
                context.translate(position.x - 0.2 + 1, position.y + 0.15 + 1.5);
                context.rotate(-0.2 + heartbeat1.time * 0.1);
                context.translate(-(position.x - 0.2 + 1), -(position.y + 0.15 + 1.5));
                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET8X, OFFSET0X, OFFSET2X, OFFSET3X, position.x - 0.2, position.y + 0.15, 2, 3);
                context.restore();

                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET4X, OFFSET0X, OFFSET4X, OFFSET4X, position.x, position.y, 4, 4);
                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET8X, OFFSET3X, OFFSET2X, OFFSET2X, position.x + 2.35, position.y + 0.77 + heartbeat1.time * 0.09, 2, 2);
            }

            // Draw the menu panel first.
            var panelPosition;
            for (i = 0, len = menuHudItems.length; i < len; i += 1) {
                if (menuHudItems[i].type === namespace.MenuTypeEnum.Panel) {
                    panelPosition = { "x": menuHudItems[i].x, "y": menuHudItems[i].y };
                    context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET0X, OFFSET5X, OFFSET4X, OFFSET3X, panelPosition.x, panelPosition.y, 4, 3);
                    break;
                }
            }

            // Draw the menu items.
            for (i = 0, len = menuHudItems.length; i < len; i += 1) {
                if (menuHudItems[i].width > 0) {

                    if (drawInteractiveMap) {
                        context.fillStyle = "rgba(255, 0, 255, 0.5)";
                        context.fillRect(menuHudItems[i].x, menuHudItems[i].y, menuHudItems[i].width, menuHudItems[i].height);
                    }

                    switch (menuHudItems[i].type) {
                        case namespace.MenuTypeEnum.ShowCredits:
                            context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET3X, OFFSET4X, OFFSET1X, OFFSET1X, menuHudItems[i].x, menuHudItems[i].y, 1, 1);
                            break;
                        case namespace.MenuTypeEnum.ToggleSound:
                            if (namespace.SoundManager.getSoundsEnabled() === 2) {
                                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET1X, menuHudItems[i].x, menuHudItems[i].y, 1, 1);
                            }
                            else if (namespace.SoundManager.getSoundsEnabled() === 1) {
                                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET8X, OFFSET6X, OFFSET1X, OFFSET1X, menuHudItems[i].x, menuHudItems[i].y, 1, 1);
                            }
                            else {
                                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET7X, OFFSET6X, OFFSET1X, OFFSET1X, menuHudItems[i].x, menuHudItems[i].y, 1, 1);
                            }
                            break;
                        case namespace.MenuTypeEnum.ShowStoryMode:
                            if (namespace.getCurrentMenuType() !== namespace.MenuTypeEnum.ShowStoryMode) {
                                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET4X, OFFSET5X, OFFSET3X, OFFSET1X, menuHudItems[i].x + 0.5, menuHudItems[i].y, 3, 1);
                            }
                            else {
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET0X, OFFSET1X, OFFSET2X, OFFSET1X, menuHudItems[i].x + 0.5, menuHudItems[i].y - 0.2, 2, 1);
                            }
                            break;
                        case namespace.MenuTypeEnum.ShowSurvivalMode:
                            if (namespace.getCurrentMenuType() !== namespace.MenuTypeEnum.ShowSurvivalMode) {
                                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET4X, OFFSET6X, OFFSET3X, OFFSET1X, menuHudItems[i].x + 0.5, menuHudItems[i].y, 3, 1);
                            }
                            else {
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET0X, OFFSET2X, OFFSET2X, OFFSET1X, menuHudItems[i].x + 0.5, menuHudItems[i].y - 0.2, 2, 1);
                            }
                            break;
                        case namespace.MenuTypeEnum.ShowOptions:
                            if (namespace.getCurrentMenuType() !== namespace.MenuTypeEnum.ShowOptions) {
                                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET4X, OFFSET7X, OFFSET3X, OFFSET1X, menuHudItems[i].x + 0.5, menuHudItems[i].y + 0.2, 3, 1);
                            }
                            else {
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET0X, OFFSET3X, OFFSET2X, OFFSET1X, menuHudItems[i].x + 0.5, menuHudItems[i].y - 0.2, 2, 1);
                            }
                            break;
                        case namespace.MenuTypeEnum.ShowMain:
                            context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET2X, OFFSET3X, OFFSET2X, OFFSET1X, menuHudItems[i].x, menuHudItems[i].y - 0.25, 2, 1);
                            break;
                        case namespace.MenuTypeEnum.SelectPrevPage:
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X, OFFSET0X, OFFSET1X, OFFSET1X, menuHudItems[i].x, menuHudItems[i].y - 0.2, 1, 1);
                            break;
                        case namespace.MenuTypeEnum.SelectNextPage:
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET1X, OFFSET0X, OFFSET1X, OFFSET1X, menuHudItems[i].x, menuHudItems[i].y - 0.2, 1, 1);
                            break;
                    }
                }
            }

            // Draw the page items.
            if (namespace.getCurrentMenuType() === namespace.MenuTypeEnum.ShowStoryMode) {
                context.save();
                context.scale(1 + namespace.getTitleHeartbeat().time * 0.03, 1 + namespace.getTitleHeartbeat().time * 0.1);
                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET3X, OFFSET0X, OFFSET2X, OFFSET1X, panelPosition.x + 0.6, panelPosition.y + 3.1, 2, 1);
                context.restore();

                directionOffset = (menuPageDirection === namespace.DirectionEnum.Right) ? menuPageTime * 10 : -(menuPageTime * 10);
                var startItem = namespace.getMenuPageIndex() * 9;
                var nextLine = 0;
                position = { "x": directionOffset + panelPosition.x + 0.4, "y": panelPosition.y + 4.6 };
                context.globalAlpha = 1 - (menuPageTime * 4);
                for (i = startItem, len = startItem + 9; i < len; i += 1) {
                    if (i > namespace.getLevelItemData().length - 1) {
                        break;
                    }
                    levelItem = namespace.getLevelItemData()[i];
                    levelItem.x = position.x;
                    levelItem.y = position.y;
                    levelItem.size = 1;

                    position.x += 1.1;

                    nextLine++;
                    if (nextLine === 3) {
                        nextLine = 0;
                        position.y += 1.1;
                        position.x = directionOffset + panelPosition.x + 0.4;
                    }

                    if (drawInteractiveMap) {
                        context.fillStyle = "rgba(255, 0, 255, 0.5)";
                        context.fillRect(levelItem.x, levelItem.y, levelItem.size, levelItem.size);
                    }

                    switch (levelItem.status) {
                        case namespace.LevelStatusEnum.Locked:
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET9X, OFFSET1X, OFFSET1X, OFFSET1X, levelItem.x, levelItem.y, 1, 1);
                            break;
                        case namespace.LevelStatusEnum.Available:
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET0X, OFFSET0X, OFFSET1X, OFFSET1X, levelItem.x, levelItem.y, 1, 1);
                            break;
                        case namespace.LevelStatusEnum.Passed:
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET8X, OFFSET1X, OFFSET1X, OFFSET1X, levelItem.x, levelItem.y, 1, 1);
                            break;
                        case namespace.LevelStatusEnum.None:
                            context.globalAlpha = context.globalAlpha / 4;
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET9X, OFFSET2X, OFFSET1X, OFFSET1X, levelItem.x, levelItem.y, 1, 1);
                            context.globalAlpha = 1 - (menuPageTime * 4);
                            break;
                    }
                    namespace.drawNumber(context, levelItem.level.toString(), levelItem.x, levelItem.y - 0.3, 0.6);
                }

                if (menuPageTime > 0 && menuPageDirection !== namespace.DirectionEnum.None) {
                    context.globalAlpha = menuPageTime * 2;
                    nextLine = 0;
                    if (menuPageDirection === namespace.DirectionEnum.Right) {
                        position.x = panelPosition.x - 4 + directionOffset;
                    }
                    else {
                        position.x = panelPosition.x + 3 + directionOffset;
                    }
                    position.y = panelPosition.y + 4.6;

                    var maxCount;
                    var movingPageIndex = namespace.getMenuPageIndex();
                    if (menuPageDirection === namespace.DirectionEnum.Right) {
                        movingPageIndex--;
                        maxCount = Math.floor(namespace.getLevelItemData().length / 9);
                        if (movingPageIndex < 0) {
                            movingPageIndex = maxCount;
                        }
                    }
                    else {
                        maxCount = Math.floor(namespace.getLevelItemData().length / 9);
                        movingPageIndex++;
                        if (movingPageIndex > maxCount) {
                            movingPageIndex = 0;
                        }
                    }

                    startItem = movingPageIndex * 9;
                    for (i = startItem; i < startItem + 9; i += 1) {
                        if (i > namespace.getLevelItemData().length - 1) {
                            break;
                        }
                        position.x += 1.1;
                        levelItem = namespace.getLevelItemData()[i];
                        switch (levelItem.status) {
                            case namespace.LevelStatusEnum.Locked:
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET9X, OFFSET1X, OFFSET1X, OFFSET1X, position.x, position.y, 1, 1);
                                break;
                            case namespace.LevelStatusEnum.Available:
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET0X, OFFSET0X, OFFSET1X, OFFSET1X, position.x, position.y, 1, 1);
                                break;
                            case namespace.LevelStatusEnum.Passed:
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET8X, OFFSET1X, OFFSET1X, OFFSET1X, position.x, position.y, 1, 1);
                                break;
                            case namespace.LevelStatusEnum.None:
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET9X, OFFSET2X, OFFSET1X, OFFSET1X, position.x, position.y, 1, 1);
                                break;
                        }
                        nextLine++;
                        if (nextLine === 3) {
                            nextLine = 0;
                            position.y += 1.1;
                            if (menuPageDirection === namespace.DirectionEnum.Right) {
                                position.x = panelPosition.x - 4 + directionOffset;
                            }
                            else {
                                position.x = panelPosition.x + 3 + directionOffset;
                            }
                        }
                    }
                }
            }
            else if (namespace.getCurrentMenuType() === namespace.MenuTypeEnum.ShowSurvivalMode) {

                pageIndex = namespace.getMenuPageIndex();
                context.save();
                context.scale(1 + namespace.getTitleHeartbeat().time * 0.03, 1 + namespace.getTitleHeartbeat().time * 0.1);
                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET6X, OFFSET0X, OFFSET2X, OFFSET1X, panelPosition.x + 0.6, panelPosition.y + 3.1, 2, 1);
                context.restore();

                directionOffset = (menuPageDirection === namespace.DirectionEnum.Right) ? menuPageTime * 10 : -(menuPageTime * 10);
                position = { "x": directionOffset + panelPosition.x + 1, "y": panelPosition.y + 4.5 };
                context.globalAlpha = 1 - (menuPageTime * 2);

                if (namespace.getMenuSurvivalItems()[pageIndex].level === 1001) {
                    context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET0X, OFFSET4X, OFFSET3X, OFFSET1X, panelPosition.x + 0.2, panelPosition.y + 6.7, 3 * 1.2, 1 * 1.2);

                    context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET8X, OFFSET0X, OFFSET2X, OFFSET1X, panelPosition.x + 3 - namespace.getTitleHeartbeat().time * 0.3, panelPosition.y + 4.7 - namespace.getTitleHeartbeat().time * 0.1, 2 + namespace.getTitleHeartbeat().time * 0.6, 1 + namespace.getTitleHeartbeat().time * 0.2);
                    var text = namespace.getSavedData().daysCountHighScore.toString();
                    var textOffset = 0;
                    if (text.length === 2) {
                        textOffset = 0.11;
                    }
                    else if (text.length === 3) {
                        textOffset = 0.22;
                    }
                    else if (text.length === 4) {
                        textOffset = 0.33;
                    }
                    namespace.drawNumber(context, text, panelPosition.x + 3.75 - textOffset, panelPosition.y + 5.3, 1);
                    context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET9X, OFFSET1X, OFFSET1X, OFFSET1X, panelPosition.x + 4 + textOffset, panelPosition.y + 5.3, 1, 1);
                }
                else if (namespace.getMenuSurvivalItems()[pageIndex].level === 1000) {
                    context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET0X, OFFSET5X, OFFSET3X, OFFSET1X, panelPosition.x + 0.2, panelPosition.y + 6.7, 3 * 1.2, 1 * 1.2);

                    context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET8X, OFFSET0X, OFFSET2X, OFFSET1X, panelPosition.x + 3 - namespace.getTitleHeartbeat().time * 0.3, panelPosition.y + 4.7 - namespace.getTitleHeartbeat().time * 0.1, 2 + namespace.getTitleHeartbeat().time * 0.6, 1 + namespace.getTitleHeartbeat().time * 0.2);
                    var text = namespace.getSavedData().killCountHighScore.toString();
                    var textOffset = 0;
                    if (text.length === 2) {
                        textOffset = 0.11;
                    }
                    else if (text.length === 3) {
                        textOffset = 0.22;
                    }
                    else if (text.length === 4) {
                        textOffset = 0.33;
                    }
                    namespace.drawNumber(context, text, panelPosition.x + 3.75 - textOffset, panelPosition.y + 5.3, 1);
                    context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET8X, OFFSET1X, OFFSET1X, OFFSET1X, panelPosition.x + 4 + textOffset, panelPosition.y + 5.3, 1, 1);
                }

                levelItem = namespace.getMenuSurvivalItems()[pageIndex];
                levelItem.x = position.x;
                levelItem.y = position.y;
                levelItem.size = 2;

                if (drawInteractiveMap) {
                    context.fillStyle = "rgba(255, 0, 255, 0.5)";
                    context.fillRect(levelItem.x, levelItem.y, levelItem.size, levelItem.size);
                }

                position.x += 1.1;

                if (namespace.getMenuSurvivalItems()[pageIndex].status === namespace.LevelStatusEnum.Locked) {
                    context.globalAlpha = Math.min(0.5, context.globalAlpha);
                }

                if (namespace.getMenuSurvivalItems()[pageIndex].level === 1001) {
                    context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET3X, OFFSET2X, OFFSET2X, OFFSET2X, levelItem.x, levelItem.y, 2, 2);
                }
                else if (namespace.getMenuSurvivalItems()[pageIndex].level === 1000) {
                    context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET5X, OFFSET2X, OFFSET2X, OFFSET2X, levelItem.x, levelItem.y, 2, 2);
                }

                if (namespace.getMenuSurvivalItems()[pageIndex].status === namespace.LevelStatusEnum.Locked) {
                    context.globalAlpha = 1 - (menuPageTime * 2);
                    context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET9X, OFFSET1X, OFFSET1X, OFFSET1X, position.x - 0.6, position.y + 0.5, 1, 1);
                }

                if (menuPageTime > 0 && menuPageDirection !== namespace.DirectionEnum.None) {
                    context.globalAlpha = menuPageTime * 2;
                    if (menuPageDirection === namespace.DirectionEnum.Right) {
                        position.x = panelPosition.x - 3 + directionOffset;
                    }
                    else {
                        position.x = panelPosition.x + 3 + directionOffset;
                    }
                    position.y = panelPosition.y + 4.5;
                    position.x += 1.1;

                    if (namespace.getMenuSurvivalItems()[pageIndex].level === 1001) {
                        context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET5X, OFFSET2X, OFFSET2X, OFFSET2X, position.x, position.y, 2, 2);
                    }
                    else if (namespace.getMenuSurvivalItems()[pageIndex].level === 1000) {
                        context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET3X, OFFSET2X, OFFSET2X, OFFSET2X, position.x, position.y, 2, 2);
                    }
                }
            }
            else if (namespace.getCurrentMenuType() === namespace.MenuTypeEnum.ShowOptions) {

                pageIndex = namespace.getMenuPageIndex();
                var smallOffset = (pageIndex + 1 < 10) ? 0.15 : 0;
                namespace.drawNumber(context, (pageIndex + 1).toString(), panelPosition.x + 1.4 + smallOffset, panelPosition.y + 3.1, 0.7);
                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET6X, OFFSET1X, OFFSET1X, OFFSET1X, panelPosition.x + 1.58, panelPosition.y + 3.4, 0.7, 0.7);
                namespace.drawNumber(context, namespace.getMenuAchievementItems().length.toString(), panelPosition.x + 2, panelPosition.y + 3.3, 0.7);

                directionOffset = (menuPageDirection === namespace.DirectionEnum.Right) ? menuPageTime * 10 : -(menuPageTime * 10);
                position = { "x": directionOffset + panelPosition.x + 1.56, "y": panelPosition.y + 4.6 };
                context.globalAlpha = 1 - (menuPageTime * 2);

                context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET1X * achievementToImageMap[pageIndex].x, OFFSET1X * achievementToImageMap[pageIndex].y, OFFSET4X, OFFSET1X, panelPosition.x - 0.55, panelPosition.y + 6.3, 4 * 1.3, 1 * 1.3);

                levelItem = namespace.getMenuAchievementItems()[pageIndex];
                levelItem.x = position.x;
                levelItem.y = position.y;
                levelItem.size = 1;

                if (drawInteractiveMap) {
                    context.fillStyle = "rgba(255, 0, 255, 0.5)";
                    context.fillRect(levelItem.x, levelItem.y, levelItem.size, levelItem.size);
                }

                position.x += 1.1;
                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET5X, OFFSET0X, OFFSET1X, OFFSET1X, levelItem.x - 0.05, levelItem.y, 1, 1);
                if (pageIndex < namespace.getSavedData().achievements.length && namespace.getSavedData().achievements[pageIndex] === namespace.LevelStatusEnum.Passed) {
                    context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET4X, OFFSET7X, OFFSET1X, OFFSET1X, levelItem.x + 0.1, levelItem.y, 1.1, 1.1);
                }

                if ((pageIndex + 1).toString().length === 1) {
                    namespace.drawNumber(context, (pageIndex + 1).toString(), levelItem.x + 0.31, levelItem.y + 0.7, 0.7);
                }
                else {
                    namespace.drawNumber(context, (pageIndex + 1).toString(), levelItem.x + 0.22, levelItem.y + 0.7, 0.7);
                }

                if (menuPageTime > 0 && menuPageDirection !== namespace.DirectionEnum.None) {
                    context.globalAlpha = menuPageTime * 2;
                    if (menuPageDirection === namespace.DirectionEnum.Right) {
                        position.x = panelPosition.x - 2.5 + directionOffset;
                    }
                    else {
                        position.x = panelPosition.x + 3 + directionOffset;
                    }
                    position.y = panelPosition.y + 4.9;
                    position.x += 1.1;
                    context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET5X, OFFSET0X, OFFSET1X, OFFSET1X, position.x, position.y, 1, 1);
                }
            }

            context.globalAlpha = 1;

            // Draw the corner zombie.
            if (menuSetupTime < 2 || namespace.getCurrentMenuType() !== namespace.MenuTypeEnum.None) {
                position = { "x": -0.7, "y": 4.56 };
                if (menuSetupTime > 1 && namespace.getCurrentMenuType() === namespace.MenuTypeEnum.None) {
                    position.y += (menuSetupTime - 1) * 2;
                    position.x -= (menuSetupTime - 1) * 2;
                }
                context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET0X, OFFSET3X, OFFSET2X, OFFSET2X, -0.5, position.y, 2, 2);
            }

            if (namespace.getShowCredits()) {
                context.save();
                context.setTransform(1, 0, 0, 1, 0, 0);
                context.drawImage(namespace.Resources.creditsScreenImage, 0, 0, canvasWidth, canvasHeight);
                context.restore();
            }

            context.drawImage(namespace.Resources.loadingScreenImage, OFFSET0X, OFFSET9X, OFFSET2X, OFFSET1X, size.x - 2, size.y - 1, 2, 1);
        }
        else if (currentGameState === namespace.GameStateEnum.GameLoading) {
            context.drawImage(namespace.Resources.loadingScreenImage, 0, 0, canvasWidth, canvasHeight);
        }
        else if (currentGameState === namespace.GameStateEnum.GameRun || currentGameState === namespace.GameStateEnum.GameLost || currentGameState === namespace.GameStateEnum.GameWon) {

            if (levelManager.showProducts) {
                if (namespace.getShowReviveScreen() === false) {
                    namespace.drawBoosters(context, levelManager);
                }
                namespace.drawProducts(context, levelManager);
            }
            else if (levelManager.showBoosters) {
                namespace.drawBoosters(context, levelManager);
            }
            else {
                // Draw the background.
                if (currentGameState === namespace.GameStateEnum.GameLost || levelManager.leftPlayer.currentWeaponCategory === namespace.WeaponCategoryEnum.Secondary || levelManager.rightPlayer.currentWeaponCategory === namespace.WeaponCategoryEnum.Secondary) {
                    context.drawImage(namespace.Resources.backgroundImage2, 0, 0, canvasWidth, canvasHeight);
                }
                else {
                    context.drawImage(namespace.Resources.backgroundImage1, 0, 0, canvasWidth, canvasHeight);
                }

                // Apply the scene cameras.
                namespace.applySceneCameras();

                // Draw the sky elements.
                context.save();
                namespace.applyVeryFarParallaxCameras();
                context.drawImage(namespace.Resources.backdrop01Image, OFFSET0X, OFFSET4X, OFFSET2X, OFFSET2X, (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2) * 0.15, levelManager.maxY - 7.9, 5, 5);
                context.restore();

                // Draw the mountains.
                if (levelManager.currentLevel < 4 || levelManager.currentLevel > 14) {
                    context.save();
                    namespace.applyFarParallaxCameras();

                    var offsetX = levelManager.minX - 30;
                    for (i = 0; i < 10; i += 1) {
                        context.drawImage(namespace.Resources.backdrop01Image, OFFSET0X, OFFSET2X, OFFSET6X, OFFSET2X, offsetX, levelManager.maxY - 7, 12, 5);
                        offsetX += 12;
                    }
                    context.restore();
                }
                else {
                    context.save();
                    namespace.applyFarParallaxCameras();

                    var offsetX = levelManager.minX - 30;
                    for (i = 0; i < 10; i += 1) {
                        context.drawImage(namespace.Resources.backdrop02Image, OFFSET0X, OFFSET4X, OFFSET6X, OFFSET2X, offsetX, levelManager.maxY - 7, 12, 5);
                        offsetX += 12;
                    }
                    context.restore();
                }

                // Draw the pillars.
                context.save();
                namespace.applyNearParallaxCameras();

                context.drawImage(namespace.Resources.backdrop01Image, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET2X, -1, levelManager.maxY - 5, 1, 4);
                context.drawImage(namespace.Resources.backdrop01Image, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET2X, 3, levelManager.maxY - 5.5, 1, 4);
                context.drawImage(namespace.Resources.backdrop01Image, OFFSET3X, OFFSET4X, OFFSET2X, OFFSET2X, 3, levelManager.maxY - 5.5, 2, 4);
                context.drawImage(namespace.Resources.backdrop01Image, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET2X, 9, levelManager.maxY - 5.5, 2, 4);
                context.drawImage(namespace.Resources.backdrop01Image, OFFSET3X, OFFSET4X, OFFSET2X, OFFSET2X, 11, levelManager.maxY - 5, 1, 4);

                context.restore();

                // Draw the junk yard.
                if (levelManager.currentLevel < 4) {
                    context.save();
                    namespace.applyVeryNearParallaxCameras();

                    var x, y;
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 6.7) * 0.75;
                    y = levelManager.maxY - 1.9;
                    context.drawImage(namespace.Resources.backdrop01Image, OFFSET6X, OFFSET2X, OFFSET1X, OFFSET1X, x, y, 1, 1);
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 6.5) * 0.75;
                    y = levelManager.maxY - 2.7;
                    context.drawImage(namespace.Resources.backdrop01Image, OFFSET7X, OFFSET4X, OFFSET3X, OFFSET2X, x, y, 3, 2);
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 5.8) * 0.75;
                    y = levelManager.maxY - 3.2;
                    context.drawImage(namespace.Resources.backdrop01Image, OFFSET7X, OFFSET2X, OFFSET3X, OFFSET2X, x, y, 3, 2);


                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 + 5.5) * 0.75;
                    y = levelManager.maxY - 2.5;
                    context.save();
                    context.translate(x * 2, 0);
                    context.scale(-1, 1);
                    context.drawImage(namespace.Resources.backdrop01Image, OFFSET7X, OFFSET2X, OFFSET3X, OFFSET2X, x, y, 3, 2);
                    context.restore();

                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 + 6) * 0.75;
                    y = levelManager.maxY - 3;
                    context.save();
                    context.translate(x * 2, 0);
                    context.scale(-1, 1);
                    context.drawImage(namespace.Resources.backdrop01Image, OFFSET7X, OFFSET4X, OFFSET3X, OFFSET2X, x, y, 3, 2);
                    context.restore();




                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 + 5.2) * 0.75;
                    y = levelManager.maxY - 2.2;
                    context.drawImage(namespace.Resources.backdrop01Image, OFFSET6X, OFFSET2X, OFFSET1X, OFFSET1X, x, y, 1, 1);
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 + 5.7) * 0.75;
                    context.drawImage(namespace.Resources.backdrop01Image, OFFSET6X, OFFSET3X, OFFSET1X, OFFSET1X, x, y, 1, 1);

                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 8.2) * 0.75;
                    y = levelManager.maxY - 2.2;
                    context.drawImage(namespace.Resources.backdrop01Image, OFFSET6X, OFFSET3X, OFFSET1X, OFFSET1X, x, y, 1, 1);
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 5) * 0.75;
                    context.drawImage(namespace.Resources.backdrop01Image, OFFSET6X, OFFSET2X, OFFSET1X, OFFSET1X, x, y, 1, 1);

                    context.restore();
                }
                else if (levelManager.currentLevel < 15) {
                    context.save();
                    namespace.applyVeryNearParallaxCameras();

                    var x, y;
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 5.7) * 0.75;
                    y = levelManager.maxY - 5;
                    context.drawImage(namespace.Resources.backdrop02Image, OFFSET0X, OFFSET0X, OFFSET8X, OFFSET4X, x, y, 8, 4);

                    y = levelManager.maxY - 3.2;
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 + 5.7) * 0.75;
                    context.drawImage(namespace.Resources.backdrop02Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, x, y, 1, 2);
                    y = levelManager.maxY - 2.5;
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 7.7) * 0.75;
                    context.drawImage(namespace.Resources.backdrop02Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, x, y, 1, 2);
                    y = levelManager.maxY - 3.2;
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 8) * 0.75;
                    context.drawImage(namespace.Resources.backdrop02Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, x, y, 1, 2);

                    context.restore();
                }
                else {
                    context.save();
                    namespace.applyVeryNearParallaxCameras();

                    var x, y;
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 5.7) * 0.75;
                    y = levelManager.maxY - 5;
                    context.drawImage(namespace.Resources.backdrop03Image, OFFSET0X, OFFSET0X, OFFSET8X, OFFSET4X, x, y, 8, 4);

                    y = levelManager.maxY - 3.2;
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 + 5.7) * 0.75;
                    context.drawImage(namespace.Resources.backdrop03Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, x, y, 1, 2);
                    y = levelManager.maxY - 2.5;
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 7.7) * 0.75;
                    context.drawImage(namespace.Resources.backdrop03Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, x, y, 1, 2);
                    y = levelManager.maxY - 3.2;
                    x = (levelManager.minX + (levelManager.maxX - levelManager.minX) / 2 - 8) * 0.75;
                    context.drawImage(namespace.Resources.backdrop03Image, OFFSET7X, OFFSET4X, OFFSET1X, OFFSET2X, x, y, 1, 2);

                    context.restore();
                }

                namespace.applyHereParallaxCameras();

                // Draw the floor.
                context.fillStyle = "rgb(180, 134, 89)";
                context.fillRect(levelManager.minX - 20.5, levelManager.maxY + 0.2, (levelManager.maxX + 20) - (levelManager.minX - 20), 30);
                var alternate = false;
                for (i = levelManager.minX - 20.5; i < levelManager.maxX + 20; i += 7) {
                    alternate = !alternate;
                    if (alternate) {
                        context.drawImage(namespace.Resources.backdrop01Image, OFFSET0X, OFFSET0X, OFFSET10X, OFFSET2X, i, levelManager.maxY - 0.7, 10, 2);
                    }
                    else {
                        context.drawImage(namespace.Resources.backdrop01Image, OFFSET0X, OFFSET0X, OFFSET10X, OFFSET2X, i, levelManager.maxY - 0.7, 10, 2);
                    }
                }

                // Draw the background effects.
                for (i = 0; i < levelManager.backgroundEffects.length; i += 1) {
                    effect = levelManager.backgroundEffects[i];
                    namespace.drawEffect(context, effect);
                }

                // draw the players stack.
                for (i = 0; i < levelManager.leftPlayer.stack.objects.length; i += 1) {
                    namespace.drawStackObject(context, levelManager.leftPlayer.stack.x, namespace.DirectionEnum.Left, levelManager.leftPlayer.stack.objects[i]);
                }

                for (i = 0; i < levelManager.rightPlayer.stack.objects.length; i += 1) {
                    namespace.drawStackObject(context, levelManager.rightPlayer.stack.x, namespace.DirectionEnum.Right, levelManager.rightPlayer.stack.objects[i]);
                }

                // Draw the stack objects.
                for (i = 0; i < levelManager.stacks.length; i += 1) {
                    var stack = levelManager.stacks[i];
                    if (namespace.DirectionEnum.None !== stack.direction) {
                        for (var u = 0; u < stack.objects.length; ++u) {
                            namespace.drawStackObject(context, stack.x, stack.direction, stack.objects[u]);
                        }
                    }
                }

                // Draw the entities.
                for (i = 0; i < levelManager.entities.length; i += 1) {
                    var entity = levelManager.entities[i];
                    namespace.drawEntity(context, entity);
                }

                // Draw the players.
                if (levelManager.rightPlayer.scaleFactor !== levelManager.leftPlayer.scaleFactor) {
                    if (levelManager.rightPlayer.scaleFactor < levelManager.leftPlayer.scaleFactor) {
                        namespace.drawPlayer(context, levelManager.rightPlayer);
                        namespace.drawPlayer(context, levelManager.leftPlayer);
                    }
                    else {
                        namespace.drawPlayer(context, levelManager.leftPlayer);
                        namespace.drawPlayer(context, levelManager.rightPlayer);
                    }
                }
                else if (levelManager.leftPlayer.direction === namespace.DirectionEnum.Right || levelManager.leftPlayer.isDead) {
                    namespace.drawPlayer(context, levelManager.rightPlayer);
                    namespace.drawPlayer(context, levelManager.leftPlayer);
                }
                else {
                    namespace.drawPlayer(context, levelManager.leftPlayer);
                    namespace.drawPlayer(context, levelManager.rightPlayer);
                }

                // Draw the effects.
                for (i = 0; i < levelManager.effects.length; i += 1) {
                    effect = levelManager.effects[i];
                    namespace.drawEffect(context, effect);
                }

                // Apply the hud cameras.
                namespace.applyHudCameras();

                var gold = namespace.getSavedData().gold;
                namespace.drawNumber(context, gold.toString(), size.x - 2, size.y - 1, 1);
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET7X, OFFSET1X, OFFSET1X, size.x - 3, size.y - 1, 1, 1);

                context.drawImage(namespace.Resources.loadingScreenImage, OFFSET0X, OFFSET7X, OFFSET3X, OFFSET2X, 0, size.y - 1, 3 * 0.7, 2 * 0.7);

                if (namespace.getConfirmCommand() !== namespace.CommandEnum.None) {
                    context.fillStyle = "rgba(0, 0, 0, 0.5)";
                    context.fillRect(0, 0, size.x, size.y);

                    switch (namespace.getConfirmCommand()) {
                        case namespace.CommandEnum.Menu:
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET0X, OFFSET7X, OFFSET5X, OFFSET1X, size.x / 2 - 2.5, size.y / 2 - 2, 5, 1);
                            break;
                        case namespace.CommandEnum.Restart:
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET5X, OFFSET7X, OFFSET3X, OFFSET1X, size.x / 2 - 1.5, size.y / 2 - 2, 3, 1);
                            break;
                    }
                    for (i = 0; i < levelHudItems.length; i += 1) {
                        if (namespace.CommandEnum.ConfirmYes === levelHudItems[i].type) {
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET9X, OFFSET4X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                        }
                        else if (namespace.CommandEnum.ConfirmNo === levelHudItems[i].type) {
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET9X, OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                        }
                    }
                }
                else {

                    if (currentGameState !== namespace.GameStateEnum.GameWon && currentGameState !== namespace.GameStateEnum.GameLost) {
                        var tutorialPosY = size.y - 2;
                        context.fillStyle = "rgba(0, 0, 0, 0.5)";
                        switch (levelManager.currentLevelStage) {
                            case namespace.LevelStageEnum.TutorialStart:
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET0X, OFFSET4X, OFFSET1X, size.x / 2 - 2, tutorialPosY, 4, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialZombiesComing:
                                if (levelManager.tutorialCooldown > 1.5) {
                                    context.fillRect(0, tutorialPosY, size.x, 1);
                                    context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET1X, OFFSET4X, OFFSET1X, size.x / 2 - 2, tutorialPosY, 4, 1);
                                }
                                else {
                                    tutorialPosY = size.y - 3;
                                    context.fillRect(0, tutorialPosY, size.x, 1);
                                    context.drawImage(namespace.Resources.tooltips2Image, OFFSET4X, OFFSET0X, OFFSET6X, OFFSET1X, size.x / 2 - 3, tutorialPosY, 6, 1);
                                }
                                break;
                            case namespace.LevelStageEnum.TutorialHitBuildButton:
                                tutorialPosY = size.y - 3;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET4X, OFFSET0X, OFFSET6X, OFFSET1X, size.x / 2 - 3, tutorialPosY, 6, 1);
                                tutorialPosY = size.y - 2.5;
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X, OFFSET6X, OFFSET1X, OFFSET1X, size.x / 2 - 1.5 - namespace.getTitleHeartbeat().time * 0.1, tutorialPosY - namespace.getTitleHeartbeat().time * 0.1, 1 + namespace.getTitleHeartbeat().time * 0.2, 1 + namespace.getTitleHeartbeat().time * 0.2);
                                break;
                            case namespace.LevelStageEnum.TutorialHitCrateButton:
                                tutorialPosY = size.y - 3;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET4X, OFFSET1X, OFFSET6X, OFFSET1X, size.x / 2 - 3, tutorialPosY, 6, 1);
                                tutorialPosY = size.y - 1.5;
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X, OFFSET6X, OFFSET1X, OFFSET1X, size.x / 2 - 1.5 - namespace.getTitleHeartbeat().time * 0.1, tutorialPosY - namespace.getTitleHeartbeat().time * 0.1, 1 + namespace.getTitleHeartbeat().time * 0.2, 1 + namespace.getTitleHeartbeat().time * 0.2);
                                break;
                            case namespace.LevelStageEnum.TutorialBuildingFirstCrate:
                                tutorialPosY = size.y - 1;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET2X, OFFSET5X, OFFSET1X, size.x / 2 - 2.5, tutorialPosY, 5, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialTapToContinue:
                                tutorialPosY = size.y - 2;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET5X, OFFSET2X, OFFSET4X, OFFSET1X, size.x / 2 - 2, tutorialPosY, 4, 1);
                                tutorialPosY = size.y - 1;
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET4X, OFFSET3X, OFFSET1X, size.x - 4, tutorialPosY, 3, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialFirstWave:
                                tutorialPosY = size.y - 2;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET7X, OFFSET7X, OFFSET1X, size.x / 2 - 3.5, tutorialPosY, 7, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialFirstKill:
                                tutorialPosY = size.y - 2;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips4Image, OFFSET2X, OFFSET1X, OFFSET7X, OFFSET1X, size.x / 2 - 3.5, tutorialPosY, 7, 1);
                                tutorialPosY = size.y - 1;
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET4X, OFFSET3X, OFFSET1X, size.x - 4, tutorialPosY, 3, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialSelectSurvivalRage:
                                tutorialPosY = size.y - 3;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips4Image, OFFSET0X, OFFSET2X, OFFSET9X, OFFSET1X, size.x / 2 - 4.5, tutorialPosY, 9, 1);
                                tutorialPosY = size.y - 1;
                                context.drawImage(namespace.Resources.tooltips4Image, OFFSET0X, OFFSET1X, OFFSET2X, OFFSET1X, size.x / 2 - 1, tutorialPosY, 2, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialFirstWaveFinishJob:
                                tutorialPosY = size.y - 2;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips4Image, OFFSET0X, OFFSET3X, OFFSET7X, OFFSET1X, size.x / 2 - 3.5, tutorialPosY, 7, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialFirstWaveDone:
                                tutorialPosY = size.y - 2;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET3X, OFFSET3X, OFFSET7X, OFFSET1X, size.x / 2 - 3.5, tutorialPosY, 7, 1);
                                tutorialPosY = size.y - 1;
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET4X, OFFSET3X, OFFSET1X, size.x - 4, tutorialPosY, 3, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialReinforceDefenses:
                                tutorialPosY = size.y - 3;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET3X, OFFSET4X, OFFSET7X, OFFSET1X, size.x / 2 - 3.5, tutorialPosY, 7, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialMoreZombiesComing:
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET5X, OFFSET7X, OFFSET1X, size.x / 2 - 3.5, tutorialPosY, 7, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialTurnRight:
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET6X, OFFSET7X, OFFSET1X, size.x / 2 - 3.5, tutorialPosY, 7, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialSecondWave:
                                tutorialPosY = size.y - 6;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET5X, OFFSET2X, OFFSET4X, OFFSET1X, size.x / 2 - 2, tutorialPosY, 4, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialSecondWaveDone:
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips3Image, OFFSET0X, OFFSET0X, OFFSET8X, OFFSET2X, size.x / 2 - 4, tutorialPosY - 0.5, 8, 2);
                                tutorialPosY = size.y - 1;
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET4X, OFFSET3X, OFFSET1X, size.x - 4, tutorialPosY, 3, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialHelpingHand:
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips3Image, OFFSET0X, OFFSET2X, OFFSET8X, OFFSET2X, size.x / 2 - 4, tutorialPosY - 0.5, 8, 2);
                                tutorialPosY = size.y - 1;
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET4X, OFFSET3X, OFFSET1X, 2, tutorialPosY, 3, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialTeamCamera:
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips3Image, OFFSET0X, OFFSET4X, OFFSET8X, OFFSET1X, size.x / 2 - 4, tutorialPosY, 8, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialTeamDefenses:
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips3Image, OFFSET0X, OFFSET6X, OFFSET8X, OFFSET1X, size.x / 2 - 4, tutorialPosY, 8, 1);
                                context.drawImage(namespace.Resources.tooltips3Image, OFFSET8X, OFFSET6X, OFFSET1X, OFFSET2X, size.x / 2 + 0.05, tutorialPosY - 0.8, 1, 2);
                                tutorialPosY = size.y - 1;
                                context.drawImage(namespace.Resources.tooltips2Image, OFFSET0X, OFFSET4X, OFFSET3X, OFFSET1X, size.x - 3, tutorialPosY, 3, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialTeamWork:
                                tutorialPosY = size.y - 5;
                                context.fillRect(0, tutorialPosY, size.x, 1);
                                context.drawImage(namespace.Resources.tooltips3Image, OFFSET0X, OFFSET5X, OFFSET8X, OFFSET1X, size.x / 2 - 4, tutorialPosY, 8, 1);
                                break;
                            case namespace.LevelStageEnum.TutorialThirdWave:
                                break;
                            case namespace.LevelStageEnum.None:
                                if (levelManager.currentLevel === 1) {
                                    tutorialPosY = size.y - 6;
                                    context.fillRect(0, tutorialPosY, size.x, 1);
                                    context.drawImage(namespace.Resources.tooltips3Image, OFFSET0X, OFFSET7X, OFFSET8X, OFFSET1X, size.x / 2 - 4, tutorialPosY, 8, 1);
                                }
                                break;
                        }

                        var posY = 1.5;
                        context.fillStyle = "rgba(0, 0, 0, 0.5)";
                        switch (levelManager.currentLevelTip) {
                            case namespace.LevelTipEnum.None:
                                break;
                            case namespace.LevelTipEnum.TipOne:
                                posY = 1.5;
                                context.drawImage(namespace.Resources.tooltips4Image, OFFSET1X, OFFSET0X, OFFSET4X, OFFSET1X, size.x / 2 - 2 - namespace.getTitleHeartbeat().time * 0.3, posY - namespace.getTitleHeartbeat().time * 0.1, 4 + namespace.getTitleHeartbeat().time * 0.6, 1 + namespace.getTitleHeartbeat().time * 0.2);
                                break;
                            case namespace.LevelTipEnum.TipTwo:
                                posY = 0.5;
                                context.drawImage(namespace.Resources.tooltips4Image, OFFSET0X, OFFSET5X, OFFSET4X, OFFSET1X, size.x / 2 - 2 - namespace.getTitleHeartbeat().time * 0.3, posY - namespace.getTitleHeartbeat().time * 0.1, 4 + namespace.getTitleHeartbeat().time * 0.6, 1 + namespace.getTitleHeartbeat().time * 0.2);
                                break;
                            case namespace.LevelTipEnum.TipThree:
                                posY = 0.5;
                                context.drawImage(namespace.Resources.tooltips4Image, OFFSET0X, OFFSET4X, OFFSET6X, OFFSET1X, size.x / 2 - 3 - namespace.getTitleHeartbeat().time * 0.3, posY - namespace.getTitleHeartbeat().time * 0.1, 6 + namespace.getTitleHeartbeat().time * 0.6, 1 + namespace.getTitleHeartbeat().time * 0.2);
                                break;
                            case namespace.LevelTipEnum.TipFive:
                                posY = 0.5;

                                if (levelManager.currentLevel < 10) {
                                    context.drawImage(namespace.Resources.tooltips1Image, OFFSET4X, OFFSET1X, OFFSET2X, OFFSET1X, size.x / 2 - 1, posY + 0.1, 2, 1);
                                    namespace.drawNumber(context, levelManager.currentLevel.toString(), size.x / 2 - 0.2, posY + 0.9, 1.1);
                                }
                                else if (levelManager.currentLevel < 100) {
                                    context.drawImage(namespace.Resources.tooltips1Image, OFFSET4X, OFFSET1X, OFFSET2X, OFFSET1X, size.x / 2 - 1, posY + 0.1, 2, 1);
                                    namespace.drawNumber(context, levelManager.currentLevel.toString(), size.x / 2 - 0.4, posY + 0.9, 1.1);
                                }
                                else if (levelManager.currentLevel === 1000) {
                                    context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET2X, size.x / 2 - 2, posY, 4, 2);
                                    context.drawImage(namespace.Resources.tooltips1Image, OFFSET4X, OFFSET2X, OFFSET3X, OFFSET2X, size.x / 2 - 1.5, posY, 3, 2);
                                }
                                else if (levelManager.currentLevel === 1001) {
                                    context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET2X, size.x / 2 - 2, posY, 4, 2);
                                    context.drawImage(namespace.Resources.tooltips1Image, OFFSET4X, OFFSET4X, OFFSET3X, OFFSET2X, size.x / 2 - 1.5, posY, 3, 2);
                                }
                                break;
                        }

                        context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET8X, OFFSET6X, OFFSET2X, OFFSET1X, -0.1, -0.1, 2, 1);
                        context.save();
                        context.translate(-levelManager.survivalPointsZoomTime, -levelManager.survivalPointsZoomTime);
                        context.scale(1 + levelManager.survivalPointsZoomTime, 1 + levelManager.survivalPointsZoomTime);
                        var number = levelManager.survivalPoints.toString();
                        if (number.length === 1) {
                            namespace.drawNumber(context, number, 0.65, 0.3, 1.2);
                        }
                        else if (number.length === 2) {
                            namespace.drawNumber(context, number, 0.5, 0.3, 1.2);
                        }
                        else if (number.length === 3) {
                            namespace.drawNumber(context, number, 0.35, 0.3, 1.2);
                        }
                        else if (number.length > 3) {
                            namespace.drawNumber(context, number, 0.2, 0.3, 1.2);
                        }
                        context.restore();

                        if (levelManager.currentLevel === 1000 || levelManager.currentLevel === 1001) {
                            if (levelManager.currentLevel === 1001) {
                                number = levelManager.daysCount.toString();
                            }
                            else {
                                number = levelManager.killCount.toString();
                            }
                        }
                        else {
                            number = "";
                        }

                        if (number !== "") {
                            var offsetTextX = 0;
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET8X, OFFSET7X, OFFSET2X, OFFSET1X, -0.1, 0.9, 2, 1);
                            if (number.length === 1) {
                                offsetTextX = 0.65;
                            }
                            else if (number.length === 2) {
                                offsetTextX = 0.5;
                            }
                            else if (number.length === 3) {
                                offsetTextX = 0.35;
                            }
                            else if (number.length > 3) {
                                offsetTextX = 0.2;
                            }

                            namespace.drawNumber(context, number, offsetTextX, 1.3, 1.2);
                            if (levelManager.currentLevel === 1000) {
                                context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET8X, OFFSET1X, OFFSET1X, OFFSET1X, 1.55 - offsetTextX, 1.3, 1, 1);
                            }
                            else {
                                context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET9X, OFFSET1X, OFFSET1X, OFFSET1X, 1.55 - offsetTextX, 1.3, 1, 1);
                            }
                        }
                    }
                    else if (currentGameState === namespace.GameStateEnum.GameWon) {
                        if (levelManager.rewardItem.won === false) {
                            context.fillStyle = "rgba(0, 0, 0, 0.5)";
                            context.fillRect(0, size.y / 2 + 1, size.x, 1);
                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET3X, OFFSET6X, OFFSET5X, OFFSET1X, size.x / 2 - 2.5, size.y / 2 + 1, 5, 1);
                        }
                    }
                    else if (currentGameState === namespace.GameStateEnum.GameLost) {
                        var imgX = size.x / 2 - 3.5;
                        var imgY = size.y / 2 - 2;
                        context.save();
                        context.translate(imgX, imgY);
                        context.rotate(-0.2);
                        context.translate(-imgX, -imgY);

                        context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET7X, OFFSET7X, OFFSET3X, OFFSET1X, imgX, imgY, 3, 1);
                        context.restore();

                        if (levelManager.currentLevel === 1000 || levelManager.currentLevel === 1001) {
                            // Print the score on screen.
                            context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET2X, size.x / 2 - 2, 1.45, 4, 2);
                            context.drawImage(namespace.Resources.effectsImage, OFFSET2X, OFFSET3X, OFFSET3X, OFFSET1X, size.x / 2 - 1.5, 1.5, 3, 1);
                            var number = (levelManager.currentLevel === 1000) ? levelManager.killCount.toString() : levelManager.daysCount.toString();
                            var offsetTextX = 0;
                            if (number.length === 1) {
                                offsetTextX = 0.3;
                            }
                            else if (number.length === 2) {
                                offsetTextX = 0.45;
                            }
                            else if (number.length === 3) {
                                offsetTextX = 0.6;
                            }
                            else if (number.length > 3) {
                                offsetTextX = 0.75;
                            }
                            namespace.drawNumber(context, number, size.x / 2 - offsetTextX, 2.4, 1.2);
                            if (levelManager.currentLevel === 1000) {
                                context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET8X, OFFSET1X, OFFSET1X, OFFSET1X, size.x / 2 + offsetTextX - 0.3, 2.4, 1, 1);
                                if (levelManager.killCount === namespace.getSavedData().killCountHighScore) {
                                    context.drawImage(namespace.Resources.effectsImage, OFFSET5X, OFFSET3X, OFFSET1X, OFFSET1X, size.x / 2 + 1, 2.5, 1, 1);
                                }
                            }
                            else {
                                context.drawImage(namespace.Resources.hud3ScreenImage, OFFSET9X, OFFSET1X, OFFSET1X, OFFSET1X, size.x / 2 + offsetTextX - 0.3, 2.4, 1, 1);
                                if (levelManager.daysCount === namespace.getSavedData().daysCountHighScore) {
                                    context.drawImage(namespace.Resources.effectsImage, OFFSET5X, OFFSET3X, OFFSET1X, OFFSET1X, size.x / 2 + 1, 2.5, 1, 1);
                                }
                            }


                        }
                    }

                    namespace.drawEntityMarkers(context, levelManager);

                    // Draw the hud objects.
                    var levelReady = (levelManager.primaryWeaponAssignmentDone && levelManager.secondaryWeaponAssignmentDone && levelManager.newItem === namespace.StackObjectTypeEnum.None);
                    for (i = 0; i < levelHudItems.length; i += 1) {
                        if ((levelHudItems[i].enabled || levelHudItems[i].x !== levelHudItems[i].tx || levelHudItems[i].y !== levelHudItems[i].ty)) {
                            if (namespace.CommandEnum.Menu === levelHudItems[i].type) {
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET3X, OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                            }
                            else if (namespace.CommandEnum.Restart === levelHudItems[i].type) {
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET3X, OFFSET4X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                            }
                            else if (namespace.CommandEnum.NextLevel === levelHudItems[i].type) {
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET0X, OFFSET6X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                            }
                            else if (namespace.CommandEnum.ToggleAdrenaline === levelHudItems[i].type) {
                                if (levelReady) {
                                    if ((levelHudItems[i].x === levelHudItems[i].tx && levelHudItems[i].y === levelHudItems[i].ty) || levelManager.survivalRageTime) {
                                        if (levelManager.survivalRageTime) {
                                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET8X, OFFSET3X, OFFSET1X, OFFSET1X, levelHudItems[i].x - namespace.getTitleHeartbeat().time * 0.15, levelHudItems[i].y - namespace.getTitleHeartbeat().time * 0.15, 1 + namespace.getTitleHeartbeat().time * 0.3, 1 + namespace.getTitleHeartbeat().time * 0.3);
                                            if (levelHudItems[i].x === levelHudItems[i].tx && levelHudItems[i].y === levelHudItems[i].ty) {
                                                namespace.drawNumber(context, levelManager.survivalRageTime.toFixed(0).toString(), levelHudItems[i].x + 0.75, levelHudItems[i].y + 0.35, 1);
                                            }
                                        }
                                        else {
                                            if (levelManager.survivalPoints >= levelManager.minSurvivalPoints) {
                                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET8X, OFFSET3X, OFFSET1X, OFFSET1X, levelHudItems[i].x - namespace.getTitleHeartbeat().time * 0.1, levelHudItems[i].y - namespace.getTitleHeartbeat().time * 0.1, 1 + namespace.getTitleHeartbeat().time * 0.2, 1 + namespace.getTitleHeartbeat().time * 0.2);
                                            }
                                            else if(levelManager.currentLevel > 2) {
                                                context.drawImage(namespace.Resources.iapImage, OFFSET5X, OFFSET1X, OFFSET1X, OFFSET1X, levelHudItems[i].x - namespace.getTitleHeartbeat().time * 0.1, levelHudItems[i].y - namespace.getTitleHeartbeat().time * 0.1, 1 + namespace.getTitleHeartbeat().time * 0.2, 1 + namespace.getTitleHeartbeat().time * 0.2);
                                            }
                                        }
                                    }
                                }
                            }
                            else if (namespace.CommandEnum.CancelAction === levelHudItems[i].type) {
                                if (levelReady) {
                                    if (levelHudItems[i].direction === namespace.DirectionEnum.Left && levelManager.leftPlayer.buildData.cooldown > 0) {
                                        context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET9X, OFFSET3X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                                    }
                                    else if (levelHudItems[i].direction === namespace.DirectionEnum.Right && levelManager.rightPlayer.buildData.cooldown > 0) {
                                        context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET9X, OFFSET3X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                                    }
                                }
                            }
                            else if (namespace.CommandEnum.SelectStackObject === levelHudItems[i].type || namespace.CommandEnum.ShowStackObjects === levelHudItems[i].type) {

                                if (levelReady) {
                                    // Draw the object.
                                    var stackObjectType = null;
                                    var buildData = (levelHudItems[i].direction === namespace.DirectionEnum.Left) ? levelManager.leftPlayer.buildData : levelManager.rightPlayer.buildData;
                                    if (namespace.CommandEnum.SelectStackObject === levelHudItems[i].type) {
                                        stackObjectType = levelHudItems[i].stackObjectType;
                                    }
                                    else {
                                        stackObjectType = buildData.stackObjectType;
                                    }

                                    var player;
                                    if (stackObjectType === namespace.StackObjectTypeEnum.RepairCrate && namespace.CommandEnum.SelectStackObject === levelHudItems[i].type) {
                                        player = (levelHudItems[i].direction === namespace.DirectionEnum.Left) ? levelManager.leftPlayer : levelManager.rightPlayer;
                                        if (levelManager.repairKits <= 0 || (!(player.stack.objects.length > 0 && player.stack.objects[0].strength < levelManager.getStackObjectStrength(player.stack.objects[0].type)))) {
                                            context.globalAlpha = 0.3;
                                        }
                                    }

                                    if (stackObjectType === namespace.StackObjectTypeEnum.MetalCrate || stackObjectType === namespace.StackObjectTypeEnum.WoodCrate) {
                                        player = (levelHudItems[i].direction === namespace.DirectionEnum.Left) ? levelManager.leftPlayer : levelManager.rightPlayer;
                                        if (player.stack.objects.length === levelManager.maxStackSize) {
                                            context.globalAlpha = 0.3;
                                        }
                                    }

                                    if ((levelHudItems[i].stackObjectType === namespace.StackObjectTypeEnum.DoubleWoodCrate || levelHudItems[i].stackObjectType === namespace.StackObjectTypeEnum.DoubleMetalCrate || levelHudItems[i].stackObjectType === namespace.StackObjectTypeEnum.MountedCrate || levelHudItems[i].stackObjectType === namespace.StackObjectTypeEnum.SpikeCrate || levelHudItems[i].stackObjectType === namespace.StackObjectTypeEnum.TNTCrate) && levelManager.survivalPoints < 40) {
                                        context.globalAlpha = 0.3;
                                    }

                                    if (stackObjectType !== namespace.StackObjectTypeEnum.None) {
                                        if (levelHudItems[i].selected) {
                                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET8X, OFFSET1X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                                        }
                                        else {
                                            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET8X, OFFSET0X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                                        }
                                    }

                                    switch (stackObjectType) {
                                        case namespace.StackObjectTypeEnum.None:
                                            context.save();
                                            if (levelHudItems[i].direction === namespace.DirectionEnum.Left) {
                                                context.translate(levelHudItems[i].x * 2 + 1, 0);
                                                context.scale(-1, 1);
                                            }

                                            if (levelHudItems[i].selected) {
                                             //   context.globalAlpha = 0.3;
                                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X, OFFSET3X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                                             //   context.globalAlpha = 1;
                                            }
                                            else {
                                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X, OFFSET2X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                                            }

                                            context.restore();
                                            break;
                                        case namespace.StackObjectTypeEnum.WoodCrate:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET0X, OFFSET0X, OFFSET1X, OFFSET1X, levelHudItems[i].x + 0.15, levelHudItems[i].y + 0.15, levelHudItems[i].size * 0.7, levelHudItems[i].size * 0.7);
                                            break;
                                        case namespace.StackObjectTypeEnum.SpikeCrate:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET3X, OFFSET1X, OFFSET1X, levelHudItems[i].x + 0.18, levelHudItems[i].y + 0.1, levelHudItems[i].size * 0.7, levelHudItems[i].size * 0.7);
                                            break;
                                        case namespace.StackObjectTypeEnum.TNTCrate:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET2X, OFFSET1X, OFFSET1X, levelHudItems[i].x + 0.15, levelHudItems[i].y + 0.15, levelHudItems[i].size * 0.7, levelHudItems[i].size * 0.7);
                                            break;
                                        case namespace.StackObjectTypeEnum.MetalCrate:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, levelHudItems[i].x + 0.15, levelHudItems[i].y + 0.15, levelHudItems[i].size * 0.7, levelHudItems[i].size * 0.7);
                                            break;
                                        case namespace.StackObjectTypeEnum.MountedCrate:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET1X, levelHudItems[i].x + 0.12, levelHudItems[i].y + 0.11, levelHudItems[i].size * 0.8, levelHudItems[i].size * 0.8);
                                            break;
                                        case namespace.StackObjectTypeEnum.RepairCrate:
                                            context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET7X, OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x + 0.1, levelHudItems[i].y + 0.1, levelHudItems[i].size * 0.8, levelHudItems[i].size * 0.8);
                                            namespace.drawNumber(context, levelManager.repairKits.toString(), levelHudItems[i].x + 0.3, levelHudItems[i].y - 0.7, 0.7);
                                            break;
                                        case namespace.StackObjectTypeEnum.DoubleMetalCrate:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET1X, OFFSET2X, OFFSET1X, OFFSET1X, levelHudItems[i].x + 0.15, levelHudItems[i].y + 0.1, levelHudItems[i].size * 0.8, levelHudItems[i].size * 0.8);
                                            break;
                                        case namespace.StackObjectTypeEnum.DoubleWoodCrate:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET0X, OFFSET2X, OFFSET1X, OFFSET1X, levelHudItems[i].x + 0.15, levelHudItems[i].y + 0.1, levelHudItems[i].size * 0.8, levelHudItems[i].size * 0.8);
                                            break;
                                    }

                                    context.globalAlpha = 1;

                                    if (namespace.CommandEnum.ShowStackObjects === levelHudItems[i].type) {
                                        context.save();
                                        context.beginPath();
                                        context.arc(levelHudItems[i].x + 0.02 + levelHudItems[i].size * 0.5, levelHudItems[i].y + levelHudItems[i].size * 0.5, levelHudItems[i].size * 0.46, 0, Math.PI * 2, true);
                                        context.closePath();
                                        context.clip();
                                        context.fillStyle = "rgba(0, 0, 0, 0.5)";
                                        var max = levelManager.getStackObjectCreationTime(buildData.stackObjectType);
                                        var height = (max - buildData.cooldown) * levelHudItems[i].size / max;
                                        context.fillRect(levelHudItems[i].x, levelHudItems[i].y + height, levelHudItems[i].size, levelHudItems[i].size);
                                        context.restore();
                                    }
                                }
                            }
                            else if (namespace.CommandEnum.AssignWeapon === levelHudItems[i].type) {
                                if (levelHudItems[i].enabled) {
                                    context.save();

                                    if (levelHudItems[i].direction === namespace.DirectionEnum.Left) {
                                        context.translate(levelHudItems[i].x * 2 + 1, 0);
                                        context.scale(-1, 1);
                                    }

                                    context.translate(0, namespace.getTitleHeartbeat().time * 0.1);

                                    context.drawImage(namespace.Resources.hud2ScreenImage, levelHudItems[i].selected ? OFFSET8X : OFFSET9X, OFFSET0X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);

                                    // Draw the object.
                                    switch (levelHudItems[i].weaponType) {
                                        case namespace.WeaponTypeEnum.Rifle:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET1X, (levelHudItems[i].selected) ? OFFSET6X : OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x - 0.1, levelHudItems[i].y - 0.1, levelHudItems[i].size * 1.2, levelHudItems[i].size * 1.2);
                                            break;
                                        case namespace.WeaponTypeEnum.ShotGun:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET2X, (levelHudItems[i].selected) ? OFFSET6X : OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x - 0.1, levelHudItems[i].y - 0.1, levelHudItems[i].size * 1.2, levelHudItems[i].size * 1.2);
                                            break;
                                        case namespace.WeaponTypeEnum.AutoRifle:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET3X, (levelHudItems[i].selected) ? OFFSET6X : OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x - 0.1, levelHudItems[i].y - 0.1, levelHudItems[i].size * 1.2, levelHudItems[i].size * 1.2);
                                            break;
                                        case namespace.WeaponTypeEnum.AutoShotGun:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET4X, (levelHudItems[i].selected) ? OFFSET6X : OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x - 0.1, levelHudItems[i].y - 0.1, levelHudItems[i].size * 1.2, levelHudItems[i].size * 1.2);
                                            break;
                                        case namespace.WeaponTypeEnum.Uzi:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET5X, (levelHudItems[i].selected) ? OFFSET6X : OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x - 0.1, levelHudItems[i].y - 0.1, levelHudItems[i].size * 1.2, levelHudItems[i].size * 1.2);
                                            break;
                                        case namespace.WeaponTypeEnum.Rpg:
                                            context.drawImage(namespace.Resources.objectsImage, OFFSET6X, (levelHudItems[i].selected) ? OFFSET6X : OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x - 0.1, levelHudItems[i].y - 0.1, levelHudItems[i].size * 1.2, levelHudItems[i].size * 1.2);
                                            break;
                                    }

                                    if (levelHudItems[i].locked) {
                                        context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET8X, OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, levelHudItems[i].size, levelHudItems[i].size);
                                    }

                                    context.restore();
                                }
                            }
                            else if (levelHudItems[i].enabled && namespace.CommandEnum.CrateSelectionDone === levelHudItems[i].type) {
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET4X, OFFSET4X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);
                            }
                            else if (levelHudItems[i].enabled && namespace.CommandEnum.RewardAccepted === levelHudItems[i].type) {
                                context.save();
                                context.translate(0, 0.5);
                                context.drawImage(namespace.Resources.tooltips5Image, OFFSET0X, OFFSET0X, OFFSET5X, OFFSET4X, levelHudItems[i].x - 2, levelHudItems[i].y - 4, 5, 4);

                                if (levelManager.rewardItem.crate !== namespace.StackObjectTypeEnum.CriticalChance) {
                                    context.drawImage(namespace.Resources.tooltips5Image, OFFSET4X, OFFSET5X, OFFSET3X, OFFSET1X, levelHudItems[i].x - 1, levelHudItems[i].y - 4.2, 3, 1);
                                }
                                else {
                                    context.drawImage(namespace.Resources.tooltips2Image, OFFSET7X, OFFSET5X, OFFSET3X, OFFSET1X, levelHudItems[i].x - 1, levelHudItems[i].y - 4.2, 3, 1);
                                }

                                switch (levelManager.rewardItem.crate) {
                                    case namespace.StackObjectTypeEnum.RepairCrate:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET5X, OFFSET0X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.hud1ScreenImage, OFFSET7X, OFFSET5X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y - 2.7, 1, 1);
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET0X, OFFSET4X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.StackObjectTypeEnum.MetalCrate:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET5X, OFFSET1X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.objectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y - 2.6, 1, 1);
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET0X, OFFSET5X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.StackObjectTypeEnum.SpikeCrate:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET5X, OFFSET2X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET3X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y - 2.7, 1, 1);
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.StackObjectTypeEnum.TNTCrate:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET5X, OFFSET3X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET2X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y - 2.6, 1, 1);
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET0X, OFFSET7X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.StackObjectTypeEnum.MountedCrate:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET5X, OFFSET4X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET1X, levelHudItems[i].x - 0.05, levelHudItems[i].y - 2.6, 1.1, 1.1);
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.StackObjectTypeEnum.Challenge1:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET8X, OFFSET6X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.7, 2, 1);
                                        context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET5X, OFFSET2X, OFFSET2X, OFFSET2X, levelHudItems[i].x - 0.15, levelHudItems[i].y - 2.8, 1.4, 1.4);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET4X, OFFSET0X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.StackObjectTypeEnum.Challenge2:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET8X, OFFSET6X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.7, 2, 1);
                                        context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET3X, OFFSET2X, OFFSET2X, OFFSET2X, levelHudItems[i].x - 0.15, levelHudItems[i].y - 2.8, 1.4, 1.4);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET4X, OFFSET0X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.StackObjectTypeEnum.CriticalChance:
                                        context.drawImage(namespace.Resources.tooltips2Image, OFFSET7X, OFFSET6X, OFFSET3X, OFFSET1X, levelHudItems[i].x - 1, levelHudItems[i].y - 3.7, 3, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET7X, OFFSET2X, OFFSET2X, OFFSET2X, levelHudItems[i].x - 0.15, levelHudItems[i].y - 2.8, 1.4, 1.4);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET7X, OFFSET1X, OFFSET3X, OFFSET1X, levelHudItems[i].x - 1, levelHudItems[i].y - 1.4, 3, 1);
                                        break;
                                    case namespace.StackObjectTypeEnum.DoubleMetalCrate:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET8X, OFFSET7X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.objectsImage, OFFSET1X, OFFSET2X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y - 2.7, 1.1, 1.1);
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.StackObjectTypeEnum.DoubleWoodCrate:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET8X, OFFSET7X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.objectsImage, OFFSET0X, OFFSET2X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y - 2.7, 1.1, 1.1);
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                }

                                switch (levelManager.rewardItem.weapon) {
                                    case namespace.WeaponTypeEnum.ShotGun:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET0X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips4Image, OFFSET2X, OFFSET6X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.45, levelHudItems[i].y - 2.6, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET3X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.WeaponTypeEnum.AutoRifle:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET2X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips4Image, OFFSET4X, OFFSET6X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.45, levelHudItems[i].y - 2.6, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET1X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.WeaponTypeEnum.AutoShotGun:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET1X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips4Image, OFFSET6X, OFFSET6X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.45, levelHudItems[i].y - 2.6, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET2X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                    case namespace.WeaponTypeEnum.Rpg:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET3X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 3.6, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips4Image, OFFSET2X, OFFSET7X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.45, levelHudItems[i].y - 2.6, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET4X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.4, 4, 1);
                                        break;
                                }
                                context.restore();
                            }
                            else if (levelHudItems[i].enabled && namespace.CommandEnum.Ready === levelHudItems[i].type) {

                                context.save();
                                context.translate(0, -namespace.getTitleHeartbeat().time * 0.2);

                                if (!levelManager.primaryWeaponAssignmentDone) {
                                    context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET5X, OFFSET4X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, 0.1, 4, 1);
                                }
                                else if (!levelManager.secondaryWeaponAssignmentDone) {
                                    context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET5X, OFFSET5X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, 0.1, 4, 1);
                                }
                                context.restore();

                                context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET2X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.5, 4, 2);
                                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET4X, (levelManager.primaryWeaponAssignmentDone || levelManager.secondaryWeaponAssignmentDone) ? OFFSET5X : OFFSET4X, OFFSET1X, OFFSET1X, levelHudItems[i].x, levelHudItems[i].y, 1, 1);

                                switch (levelManager.selectedWeaponType) {
                                    case namespace.WeaponTypeEnum.Rifle:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET5X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 1.7, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET5X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1, 4, 1);
                                        break;
                                    case namespace.WeaponTypeEnum.ShotGun:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET0X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 1.7, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET3X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1, 4, 1);
                                        break;
                                    case namespace.WeaponTypeEnum.AutoRifle:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET2X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 1.7, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET1X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1, 4, 1);
                                        break;
                                    case namespace.WeaponTypeEnum.AutoShotGun:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET1X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 1.7, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET2X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1, 4, 1);
                                        break;
                                    case namespace.WeaponTypeEnum.Uzi:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET4X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 1.7, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET0X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1, 4, 1);
                                        break;
                                    case namespace.WeaponTypeEnum.Rpg:
                                        context.drawImage(namespace.Resources.tooltips5Image, OFFSET7X, OFFSET3X, OFFSET2X, OFFSET1X, levelHudItems[i].x - 0.5, levelHudItems[i].y - 1.7, 2, 1);
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET4X, OFFSET4X, OFFSET1X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1, 4, 1);
                                        break;
                                    case namespace.WeaponTypeEnum.None:
                                        context.drawImage(namespace.Resources.tooltips1Image, OFFSET4X, OFFSET6X, OFFSET4X, OFFSET2X, levelHudItems[i].x - 1.5, levelHudItems[i].y - 1.5, 4, 2);
                                        break;
                                }
                            }
                        }
                    }
                }
            }

            if (namespace.getAchievementDialogTime() > 0) {
                namespace.drawAchievementToaster(context, size);
            }
        }
        if (namespace.getShowReviewMsg()) {
            context.save();
            context.translate(0, Math.max(0, (namespace.getMenuSetupTime() * 10) - 2));
            context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET2X, size.x / 2 - 3, size.y / 2 - 2, 6, 3);
            context.drawImage(namespace.Resources.tooltips4Image, OFFSET5X, OFFSET5X, OFFSET5X, OFFSET1X, size.x / 2 - 2.5, size.y / 2 - 1.3, 5, 1);
            context.drawImage(namespace.Resources.tooltips1Image, OFFSET8X, OFFSET6X, OFFSET2X, OFFSET2X, size.x / 2 - 2.5, size.y - 3, 2, 2);
            context.drawImage(namespace.Resources.tooltips1Image, OFFSET7X, OFFSET4X, OFFSET2X, OFFSET2X, size.x / 2 + 0.5, size.y - 3, 2, 2);
            context.restore();
        }
        else if (namespace.getShowThanksMsgTime() > 0) {
            context.globalAlpha = Math.min(1, namespace.getShowThanksMsgTime());
            context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET2X, size.x / 2 - 3, size.y / 2 - 2, 6, 3);
            context.drawImage(namespace.Resources.tooltips4Image, OFFSET6X, OFFSET4X, OFFSET4X, OFFSET1X, size.x / 2 - 2, size.y / 2 - 1.3, 4, 1);
            context.globalAlpha = 1;
        }

        if (namespace.getLockedDialogTime() > 0) {
            var dialogOffsetX = (currentGameState === namespace.GameStateEnum.GameRun) ? 0 : -2.7;
            var dialogOffsetY = (currentGameState === namespace.GameStateEnum.GameRun) ? 0 : 1.5;
            context.globalAlpha = Math.min(1, namespace.getLockedDialogTime());
            context.drawImage(namespace.Resources.tooltips1Image, OFFSET0X, OFFSET6X, OFFSET4X, OFFSET2X, size.x / 2 - 2 + dialogOffsetX, size.y / 2 - 1 + dialogOffsetY, 4, 2);
            context.drawImage(namespace.Resources.tooltips5Image, OFFSET4X, OFFSET6X, OFFSET4X, OFFSET2X, size.x / 2 - 2 + dialogOffsetX, size.y / 2 - 1 + dialogOffsetY, 4, 2);
            context.globalAlpha = 1;
        }

        if (levelManager && levelManager.showReminder) {
            var offset = Math.max(0, levelManager.showReminderTime - 3.5);
            if (levelManager.showReminderTime < 0.5) {
                context.globalAlpha = levelManager.showReminderTime * 3;
            }
            context.drawImage(namespace.Resources.iapImage, OFFSET0X, OFFSET0X, OFFSET5X, OFFSET3X, size.x - 5 * 0.7 + offset * 10, size.y / 2 - 1.5, 5 * 0.7, 3 * 0.7);
            context.globalAlpha = 1;
        }

        if (namespace.getShowReviveScreen() && levelManager && levelManager.levelEndPauseCooldown < 0.6) {
            context.drawImage(namespace.Resources.iapImage, OFFSET0X, OFFSET3X, OFFSET7X, OFFSET5X, size.x / 2 - 3.5, size.y / 2 - 2.5 + Math.max(0, levelManager.levelEndPauseCooldown - 0.3) * 10, 7, 5);
        }
    };

    namespace.drawAchievementToaster = function (context, size) {
        var maxTime = namespace.getAchievementDialogMaxTime();
        var y = -1.5;
        if (namespace.getAchievementDialogTime() > maxTime - 0.5) {
            y = -1.5 + (maxTime - namespace.getAchievementDialogTime()) * 4;
        }
        else if (namespace.getAchievementDialogTime() < 0.5) {
            y = 0.5 - (0.5 - namespace.getAchievementDialogTime()) * 4;
        }
        else {
            y = 0.5;
        }
        context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET3X, OFFSET4X, OFFSET2X, size.x / 2 - 2, y, 4, 2);

        var offsetY = 0;
        var offsetX = 0;
        switch (namespace.getAchievementId()) {
            case 0:
                offsetX = OFFSET0X;
                offsetY = OFFSET0X;
                break;
            case 1:
                offsetX = OFFSET0X;
                offsetY = OFFSET1X;
                break;
            case 2:
                offsetX = OFFSET0X;
                offsetY = OFFSET2X;
                break;
            case 3:
                offsetX = OFFSET0X;
                offsetY = OFFSET3X;
                break;
            case 4:
                offsetX = OFFSET0X;
                offsetY = OFFSET4X;
                break;
            case 5:
                offsetX = OFFSET0X;
                offsetY = OFFSET5X;
                break;
            case 6:
                offsetX = OFFSET0X;
                offsetY = OFFSET6X;
                break;
            case 7:
                offsetX = OFFSET0X;
                offsetY = OFFSET7X;
                break;
            case 8:
                offsetX = OFFSET4X;
                offsetY = OFFSET0X;
                break;
            case 9:
                offsetX = OFFSET4X;
                offsetY = OFFSET1X;
                break;
            case 10:
                offsetX = OFFSET4X;
                offsetY = OFFSET2X;
                break;
            case 11:
                offsetX = OFFSET4X;
                offsetY = OFFSET3X;
                break;
            case 12:
                offsetX = OFFSET4X;
                offsetY = OFFSET4X;
                break;
            case 13:
                offsetX = OFFSET4X;
                offsetY = OFFSET5X;
                break;
            case 14:
                offsetX = OFFSET4X;
                offsetY = OFFSET6X;
                break;
        }
        context.drawImage(namespace.Resources.hud3ScreenImage, offsetX, offsetY, OFFSET4X, OFFSET1X * 0.36, size.x / 2 - 2, y + 1.2, 4, 0.36);
    };

    namespace.drawNumber = function (context, number, x, y, scale) {
        var i, len, character, offsetX, positionX = x;

        context.save();
        context.translate(x + 0.05, y + 0.5);
        context.scale(scale, scale);
        context.translate(-(x + 0.05), -(y + 0.5));

        for (i = 0, len = number.length; i < len; i += 1) {
            character = number.charAt(i);
            offsetX = parseInt(character, 10) * OFFSET4X * 0.1;
            context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET2X + offsetX, OFFSET1X, OFFSET4X * 0.1, OFFSET1X, positionX, y, 0.4, 1);
            positionX += 0.25;
        }

        context.restore();
    };

    namespace.drawEffect = function (context, effect) {
        var globalAlpha = context.globalAlpha;

        context.save();
        if (effect.direction === namespace.DirectionEnum.Left) {
            context.translate(effect.x * 2, 0);
            context.scale(-1, 1);
        }

        // Apply rotation.
        context.translate(effect.x, effect.y);
        context.rotate(effect.angle);
        context.translate(-effect.x, -effect.y);

        if (effect.selfDestroyTime < effect.startFadeTime) {
            context.globalAlpha = effect.selfDestroyTime;
        }

        // Draw the effect.
        switch (effect.type) {
            case namespace.EffectTypeEnum.Warhead:
                context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET2X, OFFSET2X, OFFSET1X, effect.x - 1, effect.y - 0.5, 2, 1);
                break;
            case namespace.EffectTypeEnum.Bullet:
                context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET0X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.MultiBullet:
                context.drawImage(namespace.Resources.effectsImage, OFFSET5X, OFFSET1X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.HeadLimb:
                context.drawImage(namespace.Resources.effectsImage, OFFSET1X, OFFSET0X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.BodyLimb1:
                context.drawImage(namespace.Resources.effectsImage, OFFSET2X, OFFSET0X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.BodyLimb2:
                context.drawImage(namespace.Resources.effectsImage, OFFSET3X, OFFSET0X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.BloodSplatter:
                context.drawImage(namespace.Resources.effectsImage, OFFSET4X, OFFSET0X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.GunFire:
                context.drawImage(namespace.Resources.effectsImage, OFFSET7X, OFFSET0X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.CartridgeCase:
                context.drawImage(namespace.Resources.effectsImage, OFFSET7X, OFFSET0X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.SmallSmoke:
                context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.Impact:
                context.drawImage(namespace.Resources.effectsImage, OFFSET5X, OFFSET0X, OFFSET1X, OFFSET1X, effect.x - 0.5, effect.y - 0.5, 1, 1);
                break;
            case namespace.EffectTypeEnum.BigSmoke:
                if (effect.selfDestroyTime > 0.5) {
                    context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, effect.x - 1, effect.y - 1, 2, 2);
                }
                else {
                    context.drawImage(namespace.Resources.effectsImage, OFFSET1X, OFFSET1X, OFFSET2X, OFFSET2X, effect.x - 2, effect.y - 2, 4, 4);
                }
                break;
            case namespace.EffectTypeEnum.Explosion:
                if (effect.selfDestroyTime > 0.5) {
                    context.drawImage(namespace.Resources.effectsImage, OFFSET3X, OFFSET1X, OFFSET2X, OFFSET2X, effect.x - 1, effect.y - 1, 2, 2);
                }
                else {
                    context.drawImage(namespace.Resources.effectsImage, OFFSET3X, OFFSET1X, OFFSET2X, OFFSET2X, effect.x - 2, effect.y - 2, 4, 4);
                }
                break;
            case namespace.EffectTypeEnum.GainSP:
                context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET4X, OFFSET1X, OFFSET1X, effect.x - 0.5 - 1 * effect.selfDestroyTime * 0.5, effect.y - 0.5 - 1 * effect.selfDestroyTime * 0.5, 1 + 1 * effect.selfDestroyTime, 1 + 1 * effect.selfDestroyTime);
                break;
            case namespace.EffectTypeEnum.GainMoreSP:
                context.drawImage(namespace.Resources.effectsImage, OFFSET1X, OFFSET4X, OFFSET1X, OFFSET1X, effect.x - 0.5 - 1 * effect.selfDestroyTime * 0.5, effect.y - 0.5 - 1 * effect.selfDestroyTime * 0.5, 1 + 1 * effect.selfDestroyTime, 1 + 1 * effect.selfDestroyTime);
                break;
            case namespace.EffectTypeEnum.Critical:
                context.drawImage(namespace.Resources.effectsImage, OFFSET0X, OFFSET5X, OFFSET2X, OFFSET1X, effect.x - 1 - 1 * effect.selfDestroyTime * 0.5, effect.y - 0.5 - 1 * effect.selfDestroyTime * 0.5, 2 + 1 * effect.selfDestroyTime, 1 + 1 * effect.selfDestroyTime);
                break;
            case namespace.EffectTypeEnum.Checkpoint:
                context.drawImage(namespace.Resources.effectsImage, OFFSET2X, OFFSET4X, OFFSET3X, OFFSET1X, effect.x - 1.5 - 1 * effect.selfDestroyTime * 0.5, effect.y - 0.5 - 1 * effect.selfDestroyTime * 0.5, 3 + 1 * effect.selfDestroyTime, 1 + 1 * effect.selfDestroyTime);
                break;
            case namespace.EffectTypeEnum.HighScore:
                context.drawImage(namespace.Resources.effectsImage, OFFSET2X, OFFSET5X, OFFSET3X, OFFSET1X, effect.x - 1.5 - 1 * effect.selfDestroyTime * 0.5, effect.y - 0.5 - 1 * effect.selfDestroyTime * 0.5, 3 + 1 * effect.selfDestroyTime, 1 + 1 * effect.selfDestroyTime);
                break;
        }
        context.globalAlpha = globalAlpha;
        context.restore();
    };

    namespace.drawStackObject = function (context, x, direction, object) {
        if (object.visible) {
            context.save();

            // Apply rotation.
            context.translate(x, object.y);
            context.rotate(object.angle);
            context.translate(-x, -object.y);

            var left = true;
            if (direction === namespace.DirectionEnum.Right || object.direction === namespace.DirectionEnum.Right) {
                left = false;
                context.translate(x * 2, 0);
                context.scale(-1, 1);
            }

            // Draw the object.
            switch (object.type) {
                case namespace.StackObjectTypeEnum.WoodCrate:
                    if (object.strength >= 3) {
                        context.drawImage(namespace.Resources.objectsImage, OFFSET0X, OFFSET0X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1.2, 1.2);
                    }
                    else if (object.strength >= 1) {
                        context.drawImage(namespace.Resources.objectsImage, OFFSET1X, OFFSET0X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1.2, 1.2);
                    }
                    else if (object.strength >= 0) {
                        context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET0X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1.2, 1.2);
                    }
                    break;
                case namespace.StackObjectTypeEnum.SpikeCrate:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET3X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1.2, 1.2);
                    break;
                case namespace.StackObjectTypeEnum.TNTCrate:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET2X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1.2, 1.2);
                    break;
                case namespace.StackObjectTypeEnum.MetalCrate:
                    if (object.strength >= 6) {
                        context.drawImage(namespace.Resources.objectsImage, OFFSET0X, OFFSET1X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1.2, 1.2);
                    }
                    else if (object.strength >= 3) {
                        context.drawImage(namespace.Resources.objectsImage, OFFSET1X, OFFSET1X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1.2, 1.2);
                    }
                    else if (object.strength >= 0) {
                        context.drawImage(namespace.Resources.objectsImage, OFFSET2X, OFFSET1X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1.2, 1.2);
                    }
                    break;
                case namespace.StackObjectTypeEnum.MountedCrate:
                    context.save();
                    context.drawImage(namespace.Resources.objectsImage, OFFSET1X, OFFSET4X, OFFSET1X, OFFSET1X, x - 0.5, object.y - 0.5, 1, 1);

                    if (object.firingAngle !== 0) {
                        context.translate(x, object.y);
                        context.rotate(left ? object.firingAngle : Math.PI - object.firingAngle);
                        context.translate(-x, -object.y + 0.9);
                        context.translate(x * 2, 0);
                        context.scale(-1, 1);
                    }
                    context.translate(object.firingRecoilTime, 0);

                    context.drawImage(namespace.Resources.objectsImage, OFFSET0X, OFFSET3X, OFFSET2X, OFFSET1X, x - 1, object.y - 1, 2, 1);
                    context.restore();
                    break;
            }

            if (object.selected) {
                context.drawImage(namespace.Resources.objectsImage, OFFSET9X, OFFSET0X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1, 1);

                context.translate(x * 2, 0);
                context.scale(-1, 1);
                context.drawImage(namespace.Resources.objectsImage, OFFSET9X, OFFSET0X, OFFSET1X, OFFSET1X, x - 0.6, object.y - 0.6, 1, 1);
            }

            context.restore();
        }
    };

    namespace.drawPlayer = function (context, entity) {

        if (entity.weapons[namespace.WeaponCategoryEnum.Primary] !== namespace.WeaponTypeEnum.None) {
            // Prepare the context.
            // Flip vertically according to direction.
            context.save();

            var buildData = entity.buildData;
            if (buildData.cooldown > 0) {
                if (entity.buildDirection === namespace.DirectionEnum.Left) {
                    context.translate(entity.x * 2, 0);
                    context.scale(-1, 1);
                }
            }
            else {
                if (entity.direction === namespace.DirectionEnum.Left) {
                    context.translate(entity.x * 2, 0);
                    context.scale(-1, 1);
                }
            }

            context.translate(entity.x, entity.y);
            context.scale(entity.scaleFactor, entity.scaleFactor);
            context.translate(-entity.x, -entity.y - 0.15);

            if (entity.scaleFactor !== 1) {
                context.translate(0, +0.2);
            }

            // Draw the entity according to its class type.
            switch (entity.classType) {
                case namespace.EntityClassTypeEnum.HumanFemale:
                    if (entity.isDead) {
                        namespace.drawDeadHumanFemale(context, entity);
                    }
                    else {
                        namespace.drawHumanFemale(context, entity);
                    }
                    break;
                case namespace.EntityClassTypeEnum.HumanMale:
                    if (entity.isDead) {
                        namespace.drawDeadHumanMale(context, entity);
                    }
                    else {
                        namespace.drawHumanMale(context, entity);
                    }
                    break;
            }
        }

        context.restore();
    };

    namespace.drawEntity = function (context, entity) {
        // Prepare the context.
        // Flip vertically according to direction.
        context.save();
        if (entity.direction === namespace.DirectionEnum.Left) {
            context.translate(entity.x * 2, 0);
            context.scale(-1, 1);
        }

        context.translate(entity.x, entity.y);
        context.scale(entity.scaleFactor, entity.scaleFactor);
        context.translate(-entity.x, -entity.y - 0.2);

        // Draw the shadow.
        context.drawImage(namespace.Resources.entitiesImage, OFFSET0X, OFFSET7X, OFFSET1X, OFFSET1X, entity.x - entity.radius - 0.05, entity.y + entity.radius - 0.05, 1, 1);

        // Draw the entity according to its state.
        switch (entity.state) {
            case namespace.EntityStateEnum.Idle:
                namespace.drawIdleEntity(context, entity);
                break;
            case namespace.EntityStateEnum.Walking:
                namespace.drawWalkingEntity(context, entity);
                break;
        }
        context.restore();

        if (entity.drawHitPoints) {
            // Draw the hit points bar.
            context.fillStyle = "black";
            context.fillRect(entity.x - 0.15, entity.y - 1.2, 0.3, 0.1);
            context.fillStyle = "red";
            context.fillRect(entity.x - 0.15, entity.y - 1.2, entity.hitPoints * 0.3 / entity.maxHitPoints, 0.1);
        }
    };

    namespace.drawEntityMarkers = function (context, levelManager) {
        var size = namespace.screenToHud({ "x": namespace.getCanvasWidth(), "y": namespace.getCanvasHeight() });
        var countLeft = -0.5, countRight = -0.5, posX = 0;
        for (var i = levelManager.entities.length - 1; i >= 0; i -= 1) {
            var entity = levelManager.entities[i];
            if (entity.offScreenDirection !== namespace.DirectionEnum.None) {
                context.save();
                var left = (entity.offScreenDirection === namespace.DirectionEnum.Left);
                if (left) {
                    countLeft++;
                    posX = countLeft * 0.25;
                }
                else {
                    countRight++;
                    posX = size.x - countRight * 0.25;

                    context.translate(posX * 2, 0);
                    context.scale(-1, 1);
                }
                var scale = 0.8;
                context.drawImage(namespace.Resources.entitiesImage, OFFSET0X, OFFSET0X, OFFSET1X, OFFSET1X, posX, 4.13, scale, scale);
                switch (entity.headStyle) {
                    case namespace.HeadStyleEnum.Punk:
                        context.drawImage(namespace.Resources.entitiesImage, OFFSET0X, OFFSET4X, OFFSET1X, OFFSET1X, posX, 4.03, scale, scale);
                        break;
                    case namespace.HeadStyleEnum.Bandaged:
                        context.drawImage(namespace.Resources.entitiesImage, OFFSET1X, OFFSET4X, OFFSET1X, OFFSET1X, posX, 4.03, scale, scale);
                        break;
                    case namespace.HeadStyleEnum.Police:
                        context.drawImage(namespace.Resources.entitiesImage, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET1X, posX, 4.03, scale, scale);
                        break;
                    case namespace.HeadStyleEnum.Tourist:
                        context.drawImage(namespace.Resources.entitiesImage, OFFSET3X, OFFSET4X, OFFSET1X, OFFSET1X, posX, 4.03, scale, scale);
                        break;
                    case namespace.HeadStyleEnum.Bandit:
                        context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET4X, OFFSET1X, OFFSET1X, posX, 4.03, scale, scale);
                        break;
                    case namespace.HeadStyleEnum.Mexican:
                        context.drawImage(namespace.Resources.entitiesImage, OFFSET5X, OFFSET4X, OFFSET1X, OFFSET1X, posX, 4.03, scale, scale);
                        break;
                }

                context.restore();
            }
        }

        if (namespace.getTitleHeartbeat().direction === 0) {
            if (countLeft !== -0.5) {
                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET1X, OFFSET6X, OFFSET1X, OFFSET1X, 0, 3.63, 1, 1);
            }
            if (countRight !== -0.5) {
                context.save();
                context.translate(size.x * 2, 0);
                context.scale(-1, 1);
                context.drawImage(namespace.Resources.hud2ScreenImage, OFFSET1X, OFFSET6X, OFFSET1X, OFFSET1X, size.x, 3.63, 1, 1);
                context.restore();
            }
        }
    };

    namespace.drawDeadHumanMale = function (context, entity) {
        context.save();

        // Animation dependent variables.
        var animTime1 = entity.heartBeats[0].time * 0.03;

        // Entity top-left corner.
        var x = entity.x - 0.5;
        var y = entity.y - 0.5;

        // Draw the body.
        context.drawImage(namespace.Resources.entitiesImage, OFFSET8X, OFFSET2X, OFFSET2X, OFFSET2X, x, y, 2, 2);


        // Weapon top-left corner.
        x = entity.x - 1.5;
        y = entity.y - 0.5;

        // Draw the arm / weapon.

        // Apply the weapon offset.
        context.translate(entity.weaponOffset.x, entity.weaponOffset.y + 0.5);

        // If not reloading we are either resting or firing.
        if (entity.reloadCooldown === 0) {
            context.translate(entity.x, entity.y);
            if (entity.firingCooldown === 0) {
                context.rotate(-0.3);
                context.translate(-entity.x, -entity.y - 0.1);
            }
            else {
                context.rotate((entity.direction === namespace.DirectionEnum.Left && entity.firingAngle !== 0) ? Math.PI - entity.firingAngle : entity.firingAngle);
                context.translate(-entity.x, -entity.y);
            }


            if (entity.firingCooldown !== 0) {
                context.translate(-entity.firingRecoilTime, 0);
            }
        }
        else {
            context.translate(entity.x, entity.y);
            context.rotate(-entity.reloadCooldown * 7 * Math.PI);
            context.translate(-entity.x, -entity.y);
        }

        switch (entity.weapons[entity.currentWeaponCategory]) {
            case namespace.WeaponTypeEnum.Rifle:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET0X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.ShotGun:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET1X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.AutoRifle:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET2X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.AutoShotGun:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET3X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.Uzi:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET4X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.Rpg:
                if (entity.firingRecoilTime === 0 && entity.firingCooldown < 2.5) {
                    context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET2X, OFFSET2X, OFFSET1X, x + 1.2, y - animTime1 - 0.2, 2, 1 - animTime1);
                }
                context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET1X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
        }

        context.restore();
    };

    namespace.drawHumanMale = function (context, entity) {
        context.save();

        // Animation dependent variables.
        var animTime1 = entity.heartBeats[0].time * 0.03;
        var animTime2 = entity.heartBeats[2].time;
        var alternateFace = (entity.heartBeats[1].time < 1.0);
        var lookAhead = true;

        var hammerIndex = (entity.currentWeaponCategory === namespace.WeaponCategoryEnum.Secondary) ? 6 : 5;

        // Entity top-left corner.
        var x = entity.x - 0.5;
        var y = entity.y - 0.5;

        var buildData = entity.buildData;
        if (buildData.cooldown > 0) {
            // Draw the body, head and hat.
            context.drawImage(namespace.Resources.entitiesImage, OFFSET3X, OFFSET1X, OFFSET1X, OFFSET1X, x, y + 0.3, 1, 1);
            if (entity.currentWeaponCategory === namespace.WeaponCategoryEnum.Secondary) {
                context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET1X, OFFSET1X, OFFSET1X, x, y - 0.2 + entity.heartBeats[hammerIndex].time * 0.1, 1, 1);
                context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET1X, OFFSET2X, OFFSET1X, x - 0.52, y - 0.2 - 0.14 + entity.heartBeats[hammerIndex].time * 0.1, 2, 1);
            }
            else {
                context.drawImage(namespace.Resources.entitiesImage, OFFSET2X, OFFSET1X, OFFSET1X, OFFSET1X, x, y - 0.15 + entity.heartBeats[hammerIndex].time * 0.1, 1, 1);
                context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET1X, OFFSET2X, OFFSET1X, x - 0.52, y - 0.15 - 0.14 + entity.heartBeats[hammerIndex].time * 0.1, 2, 1);
            }

            // Draw the right foot.
            context.save();
            context.translate(0.25, 0.52);
            context.translate(entity.x, entity.y);
            context.rotate(0.2);
            context.translate(-entity.x, -entity.y);
            context.drawImage(namespace.Resources.entitiesImage, OFFSET7X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
            context.restore();

            // Draw the left foot.
            context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET0X, OFFSET1X, OFFSET1X, x - 0.24, y + 0.52, 1, 1);

            // Draw the hand holding the pick.
            context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET3X, OFFSET1X, OFFSET1X, x, y + 0.3 - entity.heartBeats[hammerIndex].time * 0.2, 1, 1);

            // context.translate(-0.1, 0);
            context.save();
            context.translate(entity.x + 0.9, entity.y + 0.1);
            context.rotate(entity.heartBeats[hammerIndex].time - 1);
            context.translate(-(entity.x + 0.5), -(entity.y + 0.5));
            context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET2X, OFFSET1X, OFFSET1X, x, y, 1, 1);
            context.restore();
        }
        else {
            // Draw the body.
            context.drawImage(namespace.Resources.entitiesImage, OFFSET3X, OFFSET1X, OFFSET1X, OFFSET1X, x, y + 0.3, 1, 1);

            // Draw the right foot.
            context.save();
            context.translate(0.25, 0.52);
            context.translate(entity.x, entity.y);
            context.rotate(0.2);
            context.translate(-entity.x, -entity.y);
            context.drawImage(namespace.Resources.entitiesImage, OFFSET7X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
            context.restore();

            // Draw the left foot.
            context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET0X, OFFSET1X, OFFSET1X, x - 0.24, y + 0.52, 1, 1);

            // Draw the head.
            context.save();
            context.translate((lookAhead) ? entity.x : entity.x + 0.1, entity.y - 0.1 + entity.firingRecoilTime * 0.3 - animTime2 * 0.05);
            context.rotate((lookAhead) ? 0 : 0.5);
            context.translate(-entity.x, -entity.y);
            if (entity.currentWeaponCategory === namespace.WeaponCategoryEnum.Secondary) {
              //  context.translate(entity.x, entity.y);
              //  context.scale(1.3, 1.3);
             //   context.translate(-entity.x, -entity.y);
                context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET1X, OFFSET1X, OFFSET1X, x, y - 0.15, 1, 1);
                context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET1X, OFFSET2X, OFFSET1X, x - 0.52, y - 0.15 - 0.14, 2, 1);
            }
            else {
                context.drawImage(namespace.Resources.entitiesImage, ((alternateFace) ? OFFSET1X : OFFSET0X), OFFSET1X, OFFSET1X, OFFSET1X, x, y, 1, 1);
                context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET1X, OFFSET2X, OFFSET1X, x - 0.52, y - 0.14, 2, 1);
            }
            context.restore();

            // Weapon top-left corner.
            x = entity.x - 1.5;
            y = entity.y - 0.5;

            // Draw the arm / weapon.

            // Apply the weapon offset.
            context.translate(entity.weaponOffset.x, entity.weaponOffset.y);

            // If not reloading we are either resting or firing.
            if (entity.reloadCooldown === 0) {
                context.translate(entity.x, entity.y);
                if (entity.firingCooldown === 0) {
                    context.rotate(-0.3);
                    context.translate(-entity.x, -entity.y - 0.1);
                }
                else {
                    context.rotate((entity.direction === namespace.DirectionEnum.Left && entity.firingAngle !== 0) ? Math.PI - entity.firingAngle : entity.firingAngle);
                    context.translate(-entity.x, -entity.y);
                }

                if (entity.firingCooldown !== 0) {
                    context.translate(-entity.firingRecoilTime, 0);
                }
            }
            else {
                context.translate(entity.x, entity.y);
                context.rotate(-entity.reloadCooldown * 7 * Math.PI);
                context.translate(-entity.x, -entity.y);
            }

            switch (entity.weapons[entity.currentWeaponCategory]) {
                case namespace.WeaponTypeEnum.Rifle:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET0X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.ShotGun:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET1X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.AutoRifle:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET2X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.AutoShotGun:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET3X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.Uzi:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET4X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.Rpg:
                    if (entity.firingRecoilTime === 0 && entity.firingCooldown < 2.5) {
                        context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET2X, OFFSET2X, OFFSET1X, x + 1.2, y - animTime1 - 0.2, 2, 1 - animTime1);
                    }
                    context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET1X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
            }
        }

        context.restore();
    };

    namespace.drawDeadHumanFemale = function (context, entity) {
        context.save();

        // Animation dependent variables.
        var animTime1 = entity.heartBeats[0].time * 0.03;

        // Entity top-left corner.
        var x = entity.x - 1;
        var y = entity.y - 1;

        context.drawImage(namespace.Resources.entitiesImage, OFFSET8X, OFFSET0X, OFFSET2X, OFFSET2X, x, y, 2, 2);

        // Weapon top-left corner.
        x = entity.x - 1.5;
        y = entity.y - 0.5;

        // Draw the arm / weapon.

        // Apply the weapon offset.
        context.translate(entity.weaponOffset.x, entity.weaponOffset.y + 0.5);

        // If not reloading we are either resting or firing.
        if (entity.reloadCooldown === 0) {
            context.translate(entity.x, entity.y);
            if (entity.firingCooldown === 0) {
                if (entity.weapons[entity.currentWeaponCategory] !== namespace.WeaponTypeEnum.Rpg) {
                    context.rotate(0.3 + 0.1);
                }
                else {
                    context.rotate(-0.4 + 0.1);
                }

                switch (entity.weapons[entity.currentWeaponCategory]) {
                    case namespace.WeaponTypeEnum.ShotGun:
                        context.translate(-0.2, 0.1);
                        break;
                    case namespace.WeaponTypeEnum.Rifle:
                        context.translate(-0.2, 0.2);
                        break;
                    case namespace.WeaponTypeEnum.AutoRifle:
                        context.translate(-0.3, 0.2);
                        break;
                    case namespace.WeaponTypeEnum.AutoShotGun:
                        context.translate(-0.15, 0.1);
                        break;
                    case namespace.WeaponTypeEnum.Uzi:
                        context.translate(-0.1, 0.1);
                        break;
                    case namespace.WeaponTypeEnum.Rpg:
                        // context.translate(-0.5, 0.2);
                        break;
                }
            }
            else {
                context.rotate((entity.direction === namespace.DirectionEnum.Left && entity.firingAngle !== 0) ? Math.PI - entity.firingAngle : entity.firingAngle);
            }
            context.translate(-entity.x, -entity.y);

            if (entity.firingCooldown !== 0) {
                context.translate(-entity.firingRecoilTime, 0);
            }
        }
        else {
            context.translate(entity.x, entity.y);
            context.rotate(-entity.reloadCooldown * 7 * Math.PI);
            context.translate(-entity.x, -entity.y);
        }

        switch (entity.weapons[entity.currentWeaponCategory]) {
            case namespace.WeaponTypeEnum.Rifle:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET0X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.ShotGun:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET1X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.AutoRifle:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET2X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.AutoShotGun:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET3X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.Uzi:
                context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET4X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                break;
            case namespace.WeaponTypeEnum.Rpg:
                if (entity.firingRecoilTime === 0 && entity.firingCooldown < 2.5) {
                    context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET2X, OFFSET2X, OFFSET1X, x + 1.2, y + animTime1 - 0.2, 2, 1 + animTime1);
                }
                context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET0X, OFFSET3X, OFFSET1X, x, y + animTime1, 3, 1 + animTime1);
                break;
        }

        context.restore();
    };

    namespace.drawHumanFemale = function (context, entity) {
        context.save();
        // Animation dependent variables.
        var animTime1 = entity.heartBeats[0].time * 0.03;
        var animTime2 = entity.heartBeats[2].time;
        var alternateFace = (entity.heartBeats[1].time < 1.0);

        // Entity top-left corner.
        var x = entity.x - 0.5;
        var y = entity.y - 0.75;

        var hammerIndex = (entity.currentWeaponCategory === namespace.WeaponCategoryEnum.Secondary) ? 6 : 5;

        // Translate down since the female sprite is higher.
        context.translate(0, 0.25);

        var buildData = entity.buildData;
        if (buildData.cooldown > 0) {
            // Draw the body and the head.
            context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET2X, OFFSET1X, OFFSET1X * 1.5, x, y, 1, 1.5);

            if (entity.currentWeaponCategory === namespace.WeaponCategoryEnum.Secondary) {
                context.drawImage(namespace.Resources.entitiesImage, OFFSET7X, OFFSET2X, OFFSET1X, OFFSET1X * 1.5, x, y - 0.07 - 0.42 + entity.heartBeats[hammerIndex].time * 0.05, 1, 1.5);
            }
            else {
                context.drawImage(namespace.Resources.entitiesImage, OFFSET2X, OFFSET2X, OFFSET1X, OFFSET1X * 1.5, x, y - 0.42 + entity.heartBeats[hammerIndex].time * 0.05, 1, 1.5);
            }

            x = entity.x - 0.5;
            y = entity.y - 0.5;

            // Draw the hand holding the pick.
            context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET3X, OFFSET1X, OFFSET1X, x, y + 0.05 - entity.heartBeats[hammerIndex].time * 0.2, 1, 1);

            // Draw the hand holding the hammer.
            context.save();
            context.translate(entity.x + 0.9, entity.y - 0.2);
            context.rotate(entity.heartBeats[hammerIndex].time - 1);
            context.translate(-(entity.x + 0.5), -(entity.y + 0.5));
            context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET2X, OFFSET1X, OFFSET1X, x, y, 1, 1);
            context.restore();
        }
        else {
            // Draw the body.
            context.drawImage(namespace.Resources.entitiesImage, (entity.vy < 0) ? OFFSET5X : OFFSET3X, OFFSET2X, OFFSET1X, OFFSET1X * 1.5, x, y, 1, 1.5);

            // Apply translation and rotation.
            context.save();
            context.translate(entity.x, entity.y - 0.7);
            context.rotate(-0.1 + entity.firingRecoilTime - animTime2 * 0.05);
            context.translate(-entity.x, -entity.y);

            // Draw the head.
            if (entity.currentWeaponCategory === namespace.WeaponCategoryEnum.Secondary) {
                context.drawImage(namespace.Resources.entitiesImage, OFFSET7X, OFFSET2X, OFFSET1X, OFFSET1X * 1.5, x, y - 0.07, 1, 1.5);
            }
            else {
                context.drawImage(namespace.Resources.entitiesImage, ((alternateFace) ? OFFSET1X : OFFSET0X), OFFSET2X, OFFSET1X, OFFSET1X * 1.5, x, y, 1, 1.5);
            }
            context.restore();

            // Weapon top-left corner.
            x = entity.x - 1.5;
            y = entity.y - 0.5;

            // Draw the arm / weapon.

            // Apply the weapon offset.
            context.translate(entity.weaponOffset.x, entity.weaponOffset.y);

            // If not reloading we are either resting or firing.
            if (entity.reloadCooldown === 0) {
                context.translate(entity.x, entity.y);
                if (entity.firingCooldown === 0) {
                    if (entity.weapons[entity.currentWeaponCategory] !== namespace.WeaponTypeEnum.Rpg) {
                        context.rotate(0.3 + 0.1);
                    }
                    else {
                        context.rotate(-0.4 + 0.1);
                    }

                    switch (entity.weapons[entity.currentWeaponCategory]) {
                        case namespace.WeaponTypeEnum.ShotGun:
                            context.translate(-0.2, 0.1);
                            break;
                        case namespace.WeaponTypeEnum.Rifle:
                            context.translate(-0.2, 0.2);
                            break;
                        case namespace.WeaponTypeEnum.AutoRifle:
                            context.translate(-0.3, 0.2);
                            break;
                        case namespace.WeaponTypeEnum.AutoShotGun:
                            context.translate(-0.15, 0.1);
                            break;
                        case namespace.WeaponTypeEnum.Uzi:
                            context.translate(-0.1, 0.1);
                            break;
                        case namespace.WeaponTypeEnum.Rpg:
                           // context.translate(-0.5, 0.2);
                            break;
                    }
                }
                else {
                    context.rotate((entity.direction === namespace.DirectionEnum.Left && entity.firingAngle !== 0) ? Math.PI - entity.firingAngle : entity.firingAngle);
                }
                context.translate(-entity.x, -entity.y);

                if (entity.firingCooldown !== 0) {
                    context.translate(-entity.firingRecoilTime, 0);
                }
            }
            else {
                context.translate(entity.x, entity.y);
                context.rotate(-entity.reloadCooldown * 7 * Math.PI);
                context.translate(-entity.x, -entity.y);
            }

            switch (entity.weapons[entity.currentWeaponCategory]) {
                case namespace.WeaponTypeEnum.Rifle:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET0X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.ShotGun:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET1X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.AutoRifle:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET2X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.AutoShotGun:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET3X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.Uzi:
                    context.drawImage(namespace.Resources.objectsImage, OFFSET3X, OFFSET4X, OFFSET3X, OFFSET1X, x, y - animTime1, 3, 1 - animTime1);
                    break;
                case namespace.WeaponTypeEnum.Rpg:
                    if (entity.firingRecoilTime === 0 && entity.firingCooldown < 2.5) {
                        context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET2X, OFFSET2X, OFFSET1X, x + 1.2, y + animTime1 - 0.2, 2, 1 + animTime1);
                    }
                    context.drawImage(namespace.Resources.objectsImage, OFFSET6X, OFFSET0X, OFFSET3X, OFFSET1X, x, y + animTime1, 3, 1 + animTime1);
                    break;
            }
        }
        context.restore();
    };

    namespace.drawIdleEntity = function (context, entity) {
        var woundedAngle = -entity.woundedCooldown * 1.5;

        // Animation dependent variables.
        var animTime1 = entity.heartBeats[0].time * 0.03;
        var animTime2 = entity.heartBeats[2].time;
        var animTime4 = entity.heartBeats[4].time;
        var alternateFace = (entity.heartBeats[1].direction === 1);

        // Entity top-left corner.
        var x = entity.x - 0.5;
        var y = entity.y - 0.5;

        var animTimeY = y - animTime1;
        var animTimeInc = 1 + animTime1;

        // Apply rotation.
        context.translate(entity.x, entity.y);
        context.rotate(entity.angle + animTime2 * 0.1 - 0.1);
        context.translate(-entity.x, -entity.y - 0.07);

        if (entity.headStyle === namespace.HeadStyleEnum.Police) {
            context.save();
            context.translate(0.2 + animTime2 * 0.05, 0.15);
            context.translate(entity.x, entity.y);
            context.rotate(0.9 - animTime2 * 0.2);
            context.translate(-entity.x, -entity.y);
            context.drawImage(namespace.Resources.entitiesImage, OFFSET5X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
            context.restore();
        }

        // Draw the arm.
        context.save();
        context.translate(0.07 + animTime2 * 0.05, 0.25);
        context.translate(entity.x, entity.y);
        context.rotate(-0.6 - animTime2 * 0.2);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET0X, OFFSET1X, OFFSET1X, x, animTimeY - 0.07, 1, animTimeInc);
        context.restore();

        // Draw the left hand.
        context.save();
        context.translate(0.2 + animTime2 * 0.1, 0.54 - animTime2 * 0.05);
        context.translate(entity.x, entity.y);
        context.rotate(-0.5 - animTime2);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET5X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
        context.restore();

        // Draw the right foot.
        context.save();
        context.translate(0.2, 0.67 - animTime2 * 0.05);
        context.translate(entity.x, entity.y);
        context.rotate(0.2);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET7X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
        context.restore();

        // Draw the body.
        context.save();
        context.translate(entity.x, entity.y);
        context.rotate(woundedAngle * 0.3);
        context.translate(-entity.x, -entity.y);
        context.translate(0, 0.4);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET3X, OFFSET0X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
        context.restore();

        // Draw the left foot.
        context.save();
        context.translate(-0.12, 0.66);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
        context.restore();

        // Draw the head.
        context.save();
        context.translate(entity.x, entity.y);
        context.rotate(woundedAngle * 2 + animTime2 * 0.1 - 0.25);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, ((entity.woundedCooldown !== 0) ? OFFSET2X : ((alternateFace) ? OFFSET1X : OFFSET0X)), OFFSET0X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
        context.translate(0, -0.14);
        switch (entity.headStyle) {
            case namespace.HeadStyleEnum.Punk:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET0X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Bandaged:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET1X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Police:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Tourist:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET3X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Bandit:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Mexican:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET5X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
        }
        context.restore();

        // Draw the arm.
        context.save();
        context.translate(-0.1, 0.16);
        context.translate(entity.x, entity.y);
        context.rotate(-animTime4 * 0.5 + 0.1);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET0X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
        context.restore();

        // Draw the right hand.
        context.save();
        context.translate(-0.17 + animTime4 * 0.2, 0.55);
        context.translate(entity.x, entity.y);
        context.rotate(0.15 - animTime4 * 2);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET5X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
        context.restore();
    };

    namespace.drawWalkingEntity = function (context, entity) {
        var woundedAngle = -entity.woundedCooldown * 1.5;

        // Animation dependent variables.
        var animTime1 = entity.heartBeats[0].time * 0.03;
        var animTime2 = entity.heartBeats[2].time;
        var animTime3 = entity.heartBeats[3].time;
        var animTime4 = entity.heartBeats[4].time;
        var alternateFace = (entity.heartBeats[1].direction === 1);
        var walkDirection = entity.heartBeats[2].direction;

        // Entity top-left corner.
        var x = entity.x - 0.5;
        var y = entity.y - 0.5;

        var animTimeY = y - animTime1;
        var animTimeInc = 1 + animTime1;
        var anim = 1 - animTime2;

        // Apply rotation.
        context.translate(entity.x, entity.y);
        context.rotate(entity.angle + animTime2 * 0.23 - 0.1);
        context.translate(-entity.x, -entity.y - animTime3 * 0.12 - 0.07);

        if (entity.headStyle === namespace.HeadStyleEnum.Police) {
            context.save();
            context.translate(0.2 + animTime2 * 0.05, 0.15);
            context.translate(entity.x, entity.y);
            context.rotate(1.2 - animTime2 * 0.2);
            context.translate(-entity.x, -entity.y);
            context.drawImage(namespace.Resources.entitiesImage, OFFSET5X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
            context.restore();
        }

        // Draw the arm.
        context.save();
        context.translate(0.07 + animTime2 * 0.05, 0.25);
        context.translate(entity.x, entity.y);
        context.rotate(-0.6 - animTime2 * 0.2);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET0X, OFFSET1X, OFFSET1X, x, animTimeY - 0.07, 1, animTimeInc);
        context.restore();

        // Draw the left hand.
        context.save();
        context.translate(0.2 + animTime2 * 0.1, 0.54 - animTime2 * 0.05);
        context.translate(entity.x, entity.y);
        context.rotate(-0.5 - animTime2);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET5X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
        context.restore();

        // Draw the right foot.
        context.save();
        context.translate(-0.05 + anim * 0.22, 0.66);
        context.translate(entity.x, entity.y);
        context.rotate((walkDirection === 1 && animTime2 < 0.8) ? -anim * 0.7 : 0);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
        context.restore();

        // Draw the body.
        context.save();
        context.translate(entity.x, entity.y);
        context.rotate(woundedAngle * 0.3);
        context.translate(-entity.x, -entity.y);
        context.translate(0, 0.4);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET3X, OFFSET0X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
        context.restore();

        // Draw the left foot.
        context.save();
        context.translate(-0.12 + animTime2 * 0.3, 0.66);
        context.translate(entity.x, entity.y);
        context.rotate((walkDirection === 0 && animTime2 < 0.85) ? -animTime2 * 0.9 : 0);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET6X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
        context.restore();

        // Draw the head.
        context.save();
        context.translate(entity.x, entity.y);
        context.rotate(woundedAngle * 2 + animTime2 * 0.1 - 0.25);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, ((entity.woundedCooldown !== 0) ? 2 : ((alternateFace) ? OFFSET1X : OFFSET0X)), OFFSET0X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
        context.translate(0, -0.14);
        switch (entity.headStyle) {
            case namespace.HeadStyleEnum.Punk:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET0X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Bandaged:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET1X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Police:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET2X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Tourist:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET3X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Bandit:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
            case namespace.HeadStyleEnum.Mexican:
                context.drawImage(namespace.Resources.entitiesImage, OFFSET5X, OFFSET4X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
                break;
        }
        context.restore();

        // Draw the arm.
        context.save();
        context.translate(-0.1, 0.16);
        context.translate(entity.x, entity.y);
        context.rotate(-animTime4 * 0.5 + 0.1);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET4X, OFFSET0X, OFFSET1X, OFFSET1X, x, animTimeY, 1, animTimeInc);
        context.restore();

        // Draw the right hand.
        context.save();
        context.translate(-0.17 + animTime4 * 0.2, 0.55);
        context.translate(entity.x, entity.y);
        context.rotate(0.15 - animTime4 * 2);
        context.translate(-entity.x, -entity.y);
        context.drawImage(namespace.Resources.entitiesImage, OFFSET5X, OFFSET0X, OFFSET1X, OFFSET1X, x, y, 1, 1);
        context.restore();
    };

})(window.ZCJGame = window.ZCJGame || {});
